@@ -1,3 +1,8 @@
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
 #ifndef MUON_ARGS_H
 #define MUON_ARGS_H
 
@@ -8,7 +13,6 @@ struct args {
 	uint32_t len;
 };
 
-void shell_escape(struct workspace *wk, struct sbuf *sb, const char *str);
 void ninja_escape(struct workspace *wk, struct sbuf *sb, const char *str);
 void pkgconf_escape(struct workspace *wk, struct sbuf *sb, const char *str);
 
