@@ -1,3 +1,8 @@
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
 #ifndef MUON_BACKEND_COMMON_ARGS_H
 #define MUON_BACKEND_COMMON_ARGS_H
 
@@ -27,5 +32,6 @@ void setup_linker_args(struct workspace *wk, const struct project *proj,
 void setup_compiler_args_includes(struct workspace *wk, obj compiler, obj include_dirs, obj args, bool relativize);
 
 void relativize_paths(struct workspace *wk, obj arr, bool relativize_strings, obj *res);
+void relativize_path(struct workspace *wk, obj path, bool relativize_strings, obj *res);
 void relativize_path_push(struct workspace *wk, obj path, obj arr);
 #endif
