@@ -1,3 +1,8 @@
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
 #ifndef MUON_BACKEND_NINJA_H
 #define MUON_BACKEND_NINJA_H
 
@@ -10,5 +15,5 @@ struct write_tgt_ctx {
 };
 
 bool ninja_write_all(struct workspace *wk);
-int ninja_run(const char *argstr, uint32_t argstr_argc, const char *chdir, const char *capture);
+int ninja_run(struct workspace *wk, obj args, const char *chdir, const char *capture);
 #endif
