@@ -1,3 +1,8 @@
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
 #ifndef MUON_CMD_TEST_H
 #define MUON_CMD_TEST_H
 #include <stdbool.h>
@@ -27,5 +32,5 @@ struct test_options {
 	enum test_category cat;
 };
 
-bool tests_run(struct test_options *opts);
+bool tests_run(struct test_options *opts, const char *argv0);
 #endif
