@@ -1,3 +1,8 @@
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
 #ifndef MUON_FUNCTIONS_COMMON_H
 #define MUON_FUNCTIONS_COMMON_H
 
@@ -44,7 +49,7 @@ bool interp_args(struct workspace *wk, uint32_t args_node,
 	struct args_kw keyword_args[]);
 bool builtin_run(struct workspace *wk, bool have_rcvr, obj rcvr_id, uint32_t node_id, obj *res);
 
-bool analyze_function(struct workspace *wk, const struct func_impl_name *fi, uint32_t args_node, obj rcvr, obj *res);
+bool analyze_function(struct workspace *wk, const struct func_impl_name *fi, uint32_t args_node, obj rcvr, obj *res, bool *was_pure);
 
 void dump_function_signatures(struct workspace *wk);
 #endif
