@@ -1,6 +1,14 @@
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-FileCopyrightText: illiliti <illiliti@dimension.sh>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
 #ifndef MUON_LANG_OBJECT_H
 #define MUON_LANG_OBJECT_H
 
+#include "compat.h"
+
 #include <stdarg.h>
 #include <stdio.h>
 
@@ -297,6 +305,7 @@ struct obj_custom_target {
 	obj depends; // obj_array
 	obj private_path; // obj_string
 	obj env; // str | list[str] | dict[str] | env
+	obj depfile; // str
 	enum custom_target_flags flags;
 };
 
@@ -310,6 +319,7 @@ enum dependency_type {
 	dependency_type_pkgconf,
 	dependency_type_threads,
 	dependency_type_external_library,
+	dependency_type_appleframeworks,
 };
 
 enum dep_flags {
@@ -335,7 +345,7 @@ struct obj_dependency {
 };
 
 struct obj_external_program {
-	bool found;
+	bool found, guessed_ver;
 	obj full_path;
 	obj ver;
 };
@@ -428,6 +438,11 @@ enum build_option_type {
 	build_option_type_count,
 };
 
+enum build_option_kind {
+	build_option_kind_default,
+	build_option_kind_prefixed_dir,
+};
+
 enum option_value_source {
 	option_value_source_unset,
 	option_value_source_default,
@@ -450,6 +465,7 @@ struct obj_option {
 	obj description;
 	enum option_value_source source;
 	enum build_option_type type;
+	enum build_option_kind kind;
 	bool yield, builtin;
 };
 
@@ -540,17 +556,18 @@ bool obj_clone(struct workspace *wk_src, struct workspace *wk_dest, obj val, obj
 
 bool obj_vasprintf(struct workspace *wk, struct sbuf *sb, const char *fmt, va_list ap);
 bool obj_asprintf(struct workspace *wk, struct sbuf *sb, const char *fmt, ...)
-__attribute__ ((format(printf, 3, 4)));
+MUON_ATTR_FORMAT(printf, 3, 4);
 bool obj_vfprintf(struct workspace *wk, FILE *f, const char *fmt, va_list ap)
-__attribute__ ((format(printf, 3, 0)));
+MUON_ATTR_FORMAT(printf, 3, 0);
 bool obj_fprintf(struct workspace *wk, FILE *f, const char *fmt, ...)
-__attribute__ ((format(printf, 3, 4)));
+MUON_ATTR_FORMAT(printf, 3, 4);
 bool obj_printf(struct workspace *wk, const char *fmt, ...)
-__attribute__ ((format(printf, 2, 3)));
+MUON_ATTR_FORMAT(printf, 2, 3);
 void obj_inspect(struct workspace *wk, FILE *out, obj val);
 
 typedef enum iteration_result (*obj_array_iterator)(struct workspace *wk, void *ctx, obj val);
 void obj_array_push(struct workspace *wk, obj arr, obj child);
+void obj_array_prepend(struct workspace *wk, obj *arr, obj val);
 bool obj_array_foreach(struct workspace *wk, obj arr, void *ctx, obj_array_iterator cb);
 bool obj_array_foreach_flat(struct workspace *wk, obj arr, void *usr_ctx, obj_array_iterator cb);
 bool obj_array_in(struct workspace *wk, obj arr, obj val);
@@ -560,6 +577,7 @@ void obj_array_extend(struct workspace *wk, obj arr, obj arr2);
 void obj_array_extend_nodup(struct workspace *wk, obj arr, obj arr2);
 void obj_array_dup(struct workspace *wk, obj arr, obj *res);
 bool obj_array_join(struct workspace *wk, bool flat, obj arr, obj join, obj *res);
+void obj_array_tail(struct workspace *wk, obj arr, obj *res);
 void obj_array_set(struct workspace *wk, obj arr, int64_t i, obj v);
 void obj_array_del(struct workspace *wk, obj arr, int64_t i);
 void obj_array_dedup(struct workspace *wk, obj arr, obj *res);
