@@ -1,3 +1,8 @@
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
 #ifndef MUON_PLATFORM_PATH_H
 #define MUON_PLATFORM_PATH_H
 #include <stdbool.h>
@@ -5,6 +10,18 @@
 
 #define PATH_SEP '/'
 
+#ifdef _WIN32
+#define ENV_PATH_SEP ';'
+#else
+#define ENV_PATH_SEP ':'
+#endif
+
+#ifdef _WIN32
+#define ENV_PATH_SEP_STR ";"
+#else
+#define ENV_PATH_SEP_STR ":"
+#endif
+
 struct workspace;
 struct sbuf;
 
@@ -32,4 +49,6 @@ void path_basename(struct workspace *wk, struct sbuf *buf, const char *path);
 void path_dirname(struct workspace *wk, struct sbuf *buf, const char *path);
 void path_executable(struct workspace *wk, struct sbuf *buf, const char *path);
 void _path_normalize(struct workspace *wk, struct sbuf *buf, bool optimize);
+void path_to_posix(char *path);
+void shell_escape(struct workspace *wk, struct sbuf *sb, const char *str);
 #endif
