@@ -1,7 +1,12 @@
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
 #ifndef MUON_WRAP_H
 #define MUON_WRAP_H
 
-#include "posix.h"
+#include "compat.h"
 
 #include <stdbool.h>
 #include <stdint.h>
