@@ -1,4 +1,9 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include "args.h"
 #include "backend/backend.h"
@@ -49,6 +54,7 @@ backend_output(struct workspace *wk)
 {
 	TracyCZoneAutoS;
 	if (!ninja_write_all(wk)) {
+		LOG_E("backend output failed");
 		TracyCZoneAutoE;
 		return false;
 	}
