@@ -1,4 +1,9 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <errno.h>
 #include <string.h>
@@ -808,10 +813,7 @@ run_project_tests(struct workspace *wk, void *_ctx, obj proj_name, obj arr)
 	if (get_obj_array(wk, ctx->deps)->len && !ctx->opts->no_rebuild) {
 		obj ninja_cmd;
 		obj_array_dedup(wk, ctx->deps, &ninja_cmd);
-		const char *argstr;
-		uint32_t argc;
-		join_args_argstr(wk, &argstr, &argc, ninja_cmd);
-		if (ninja_run(argstr, argc, NULL, NULL) != 0) {
+		if (ninja_run(wk, ninja_cmd, NULL, NULL) != 0) {
 			LOG_W("failed to run ninja");
 		}
 	}
@@ -843,44 +845,42 @@ run_project_tests(struct workspace *wk, void *_ctx, obj proj_name, obj arr)
 }
 
 bool
-tests_run(struct test_options *opts)
+tests_run(struct test_options *opts, const char *argv0)
 {
 	bool ret = false;
 	SBUF_manual(tests_src);
 
 	struct workspace wk;
 	workspace_init_bare(&wk);
+	wk.argv0 = argv0;
 
 	if (!opts->jobs) {
 		opts->jobs = 4;
 	}
 
-	{
-		obj ninja_cmd;
-		make_obj(&wk, &ninja_cmd, obj_array);
-		obj_array_push(&wk, ninja_cmd, make_str(&wk, "build.ninja"));
-		const char *argstr;
-		uint32_t argc;
-		join_args_argstr(&wk, &argstr, &argc, ninja_cmd);
-		ninja_run(argstr, argc, NULL, NULL);
-	}
-
-	path_join(NULL, &tests_src, output_path.private_dir, output_path.tests);
-
-	FILE *f;
-	f = fs_fopen(tests_src.buf, "r");
-
-	sbuf_destroy(&tests_src);
-
-	if (!f) {
-		goto ret;
-	}
-
 	struct run_test_ctx ctx = {
 		.opts = opts,
 		.setup = { .timeout_multiplier = 1.0f, },
 	};
 
+	darr_init(&ctx.test_results, 32, sizeof(struct test_result));
+	ctx.jobs = z_calloc(ctx.opts->jobs, sizeof(struct test_result));
+
+	{ // load global opts
+		obj option_info;
+		if (!serial_load_from_private_dir(&wk, &option_info, output_path.option_info)) {
+			goto ret;
+		}
+		obj_array_index(&wk, option_info, 0, &wk.global_opts);
+	}
+
+	{
+		obj ninja_cmd;
+		make_obj(&wk, &ninja_cmd, obj_array);
+		obj_array_push(&wk, ninja_cmd, make_str(&wk, "build.ninja"));
+		ninja_run(&wk, ninja_cmd, NULL, NULL);
+	}
+
 	{
 		int fd;
 		if (!fs_fileno(log_file(), &fd)) {
@@ -889,7 +889,7 @@ tests_run(struct test_options *opts)
 
 		if (opts->display == test_display_auto) {
 			opts->display = test_display_dots;
-			if (term_isterm(fd)) {
+			if (fs_is_a_tty_from_fd(fd)) {
 				opts->display = test_display_bar;
 			}
 		}
@@ -905,14 +905,8 @@ tests_run(struct test_options *opts)
 		}
 	}
 
-	darr_init(&ctx.test_results, 32, sizeof(struct test_result));
-	ctx.jobs = z_calloc(ctx.opts->jobs, sizeof(struct test_result));
-
 	obj tests_dict;
-	if (!serial_load(&wk, &tests_dict, f)) {
-		LOG_E("invalid data file");
-		goto ret;
-	} else if (!fs_fclose(f)) {
+	if (!serial_load_from_private_dir(&wk, &tests_dict, output_path.tests)) {
 		goto ret;
 	}
 
