@@ -1,4 +1,12 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-FileCopyrightText: illiliti <illiliti@dimension.sh>
+ * SPDX-FileCopyrightText: Owen Rafferty <owen@owenrafferty.com>
+ * SPDX-FileCopyrightText: illiliti <illiliti@thunix.net>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -7,10 +15,12 @@
 #include "buf_size.h"
 #include "compilers.h"
 #include "error.h"
+#include "functions/machine.h"
 #include "guess.h"
 #include "lang/workspace.h"
 #include "log.h"
 #include "options.h"
+#include "platform/path.h"
 #include "platform/run_cmd.h"
 
 const char *
@@ -22,6 +32,8 @@ compiler_type_to_s(enum compiler_type t)
 	case compiler_clang: return "clang";
 	case compiler_apple_clang: return "clang";
 	case compiler_clang_llvm_ir: return "clang";
+	case compiler_nasm: return "nasm";
+	case compiler_yasm: return "yasm";
 	case compiler_type_count: UNREACHABLE;
 	}
 
@@ -51,6 +63,7 @@ static const char *compiler_language_names[compiler_language_count] = {
 	[compiler_language_objc] = "objc",
 	[compiler_language_assembly] = "assembly",
 	[compiler_language_llvm_ir] = "llvm_ir",
+	[compiler_language_nasm] = "nasm",
 };
 
 const char *
@@ -83,6 +96,7 @@ static const char *compiler_language_exts[compiler_language_count][10] = {
 	[compiler_language_objc] = { "m", "mm", "M" },
 	[compiler_language_assembly] = { "S" },
 	[compiler_language_llvm_ir] = { "ll" },
+	[compiler_language_nasm] = { "asm" },
 };
 
 bool
@@ -128,8 +142,10 @@ coalesce_link_languages(enum compiler_language cur, enum compiler_language new)
 			return compiler_language_assembly;
 		}
 		break;
+	case compiler_language_nasm:
 	case compiler_language_c:
 	case compiler_language_c_obj:
+	case compiler_language_objc:
 		if (!cur) {
 			return compiler_language_c;
 		}
@@ -141,7 +157,6 @@ coalesce_link_languages(enum compiler_language cur, enum compiler_language new)
 			return compiler_language_cpp;
 		}
 		break;
-	case compiler_language_objc:
 	case compiler_language_count:
 		UNREACHABLE;
 	}
@@ -230,6 +245,70 @@ detection_over:
 	return true;
 }
 
+static bool
+compiler_detect_nasm(struct workspace *wk, obj cmd_arr, obj *comp_id)
+{
+	struct run_cmd_ctx cmd_ctx = { 0 };
+	if (!run_cmd_arr(wk, &cmd_ctx, cmd_arr, "--version")) {
+		run_cmd_ctx_destroy(&cmd_ctx);
+		return false;
+	}
+
+	enum compiler_type type;
+	obj ver;
+
+	if (strstr(cmd_ctx.out.buf, "NASM")) {
+		type = compiler_nasm;
+	} else if (strstr(cmd_ctx.out.buf, "yasm")) {
+		type = compiler_yasm;
+	} else {
+		// Just assume it is nasm
+		type = compiler_nasm;
+	}
+
+	if (!guess_version(wk, cmd_ctx.out.buf, &ver)) {
+		ver = make_str(wk, "unknown");
+	}
+
+	obj new_cmd;
+	obj_array_dup(wk, cmd_arr, &new_cmd);
+
+	{
+		uint32_t addr_bits = machine_cpu_address_bits();
+		enum machine_system sys = machine_system();
+
+		const char *plat;
+		SBUF(define);
+
+		if (sys == machine_system_windows || sys == machine_system_cygwin) {
+			plat = "win";
+			sbuf_pushf(wk, &define, "WIN%d", addr_bits);
+		} else if (sys == machine_system_darwin) {
+			plat = "macho";
+			sbuf_pushs(wk, &define, "MACHO");
+		} else {
+			plat = "elf";
+			sbuf_pushs(wk, &define, "ELF");
+		}
+
+		obj_array_push(wk, new_cmd, make_strf(wk, "-f%s%d", plat, addr_bits));
+		obj_array_push(wk, new_cmd, make_strf(wk, "-D%s", define.buf));
+		if (addr_bits == 64) {
+			obj_array_push(wk, new_cmd, make_str(wk, "-D__x86_64__"));
+		}
+	}
+
+	make_obj(wk, comp_id, obj_compiler);
+	struct obj_compiler *comp = get_obj_compiler(wk, *comp_id);
+	comp->cmd_arr = new_cmd;
+	comp->type = type;
+	comp->ver = ver;
+	comp->lang = compiler_language_nasm;
+
+	run_cmd_ctx_destroy(&cmd_ctx);
+	return true;
+}
+
 static bool
 compiler_get_libdirs(struct workspace *wk, struct obj_compiler *comp)
 {
@@ -256,7 +335,7 @@ compiler_get_libdirs(struct workspace *wk, struct obj_compiler *comp)
 				.len = e ? (uint32_t)(e - s) : strlen(s),
 			};
 
-			comp->libdirs = str_split(wk, &str, &WKSTR(":"));
+			comp->libdirs = str_split(wk, &str, &WKSTR(ENV_PATH_SEP_STR));
 			goto done;
 		}
 
@@ -288,17 +367,24 @@ done:
 bool
 compiler_detect(struct workspace *wk, obj *comp, enum compiler_language lang)
 {
-	static const char *compiler_option[] = {
+	static const char *compiler_option[compiler_language_count] = {
 		[compiler_language_c] = "env.CC",
 		[compiler_language_cpp] = "env.CXX",
+		[compiler_language_objc] = "env.OBJC",
+		[compiler_language_nasm] = "env.NASM",
 	};
 
+	if (!compiler_option[lang]) {
+		return false;
+	}
+
 	obj cmd_arr;
 	get_option_value(wk, NULL, compiler_option[lang], &cmd_arr);
 
 	switch (lang) {
 	case compiler_language_c:
 	case compiler_language_cpp:
+	case compiler_language_objc:
 		if (!compiler_detect_c_or_cpp(wk, cmd_arr, comp)) {
 			return false;
 		}
@@ -307,6 +393,11 @@ compiler_detect(struct workspace *wk, obj *comp, enum compiler_language lang)
 		compiler_get_libdirs(wk, compiler);
 		compiler->lang = lang;
 		return true;
+	case compiler_language_nasm:
+		if (!compiler_detect_nasm(wk, cmd_arr, comp)) {
+			return false;
+		}
+		return true;
 	default:
 		LOG_E("tried to get a compiler for unsupported language '%s'", compiler_language_to_s(lang));
 		return false;
@@ -362,6 +453,7 @@ compiler_posix_args_optimization(uint32_t lvl)
 	case compiler_optimization_lvl_3:
 		argv[0] = "-O1";
 		break;
+	case compiler_optimization_lvl_none:
 	case compiler_optimization_lvl_g:
 	case compiler_optimization_lvl_s:
 		args.len = 0;
@@ -436,6 +528,9 @@ compiler_gcc_args_optimization(uint32_t lvl)
 	COMPILER_ARGS({ NULL });
 
 	switch ((enum compiler_optimization_lvl)lvl) {
+	case compiler_optimization_lvl_none:
+		args.len = 0;
+		break;
 	case compiler_optimization_lvl_0:
 		argv[0] = "-O0";
 		break;
@@ -581,10 +676,34 @@ compiler_gcc_args_visibility(uint32_t type)
 	return &args;
 }
 
-compiler_get_arg_func_0 as_needed;
-compiler_get_arg_func_0 no_undefined;
-compiler_get_arg_func_0 start_group;
-compiler_get_arg_func_0 end_group;
+static const struct args *
+compiler_gcc_args_specify_lang(const char *language)
+{
+	COMPILER_ARGS({ "-x", NULL, });
+
+	argv[1] = language;
+
+	return &args;
+}
+
+static const struct args *
+compiler_gcc_args_color_output(const char *when)
+{
+	static char buf[BUF_SIZE_S];
+	COMPILER_ARGS({ buf });
+
+	snprintf(buf, BUF_SIZE_S, "-fdiagnostics-color=%s", when);
+
+	return &args;
+}
+
+static const struct args *
+compiler_gcc_args_lto(void)
+{
+	COMPILER_ARGS({ "-flto" });
+
+	return &args;
+}
 
 static const struct args *
 compiler_arg_empty_0(void)
@@ -654,6 +773,9 @@ build_compilers(void)
 			.sanitize        = compiler_arg_empty_1s,
 			.define          = compiler_arg_empty_1s,
 			.visibility      = compiler_arg_empty_1i,
+			.specify_lang    = compiler_arg_empty_1s,
+			.color_output    = compiler_arg_empty_1s,
+			.enable_lto      = compiler_arg_empty_0,
 		}
 	};
 
@@ -685,6 +807,9 @@ build_compilers(void)
 	gcc.args.pie = compiler_gcc_args_pie;
 	gcc.args.sanitize = compiler_gcc_args_sanitize;
 	gcc.args.visibility = compiler_gcc_args_visibility;
+	gcc.args.specify_lang = compiler_gcc_args_specify_lang;
+	gcc.args.color_output = compiler_gcc_args_color_output;
+	gcc.args.enable_lto = compiler_gcc_args_lto;
 	gcc.deps = compiler_deps_gcc;
 	gcc.linker = linker_gcc;
 
@@ -696,6 +821,18 @@ build_compilers(void)
 	compilers[compiler_clang] = gcc;
 	compilers[compiler_apple_clang] = apple_clang;
 	compilers[compiler_clang_llvm_ir] = clang_llvm_ir;
+
+	struct compiler nasm = empty;
+	nasm.args.output = compiler_posix_args_output;
+	nasm.args.optimization = compiler_posix_args_optimization;
+	nasm.args.debug = compiler_posix_args_debug;
+	nasm.args.include = compiler_posix_args_include;
+	nasm.args.include_system = compiler_posix_args_include;
+	nasm.args.define = compiler_posix_args_define;
+	nasm.linker = linker_posix;
+
+	compilers[compiler_nasm] = nasm;
+	compilers[compiler_yasm] = nasm;
 }
 
 static const struct args *
@@ -844,6 +981,7 @@ build_linkers(void)
 	gcc.args.fatal_warnings = linker_gcc_args_fatal_warnings;
 	gcc.args.whole_archive = linker_gcc_args_whole_archive;
 	gcc.args.no_whole_archive = linker_gcc_args_no_whole_archive;
+	gcc.args.enable_lto = compiler_gcc_args_lto;
 
 	struct linker apple = posix;
 
@@ -858,3 +996,48 @@ compilers_init(void)
 	build_compilers();
 	build_linkers();
 }
+
+enum ar_type {
+	ar_posix,
+	ar_gcc,
+};
+
+static enum ar_type
+compiler_detect_ar_type(void)
+{
+	struct run_cmd_ctx cmd_ctx = { 0 };
+	if (!run_cmd_argv(&cmd_ctx, (char *[]){ "ar", "--version", NULL }, NULL, 0)) {
+		run_cmd_ctx_destroy(&cmd_ctx);
+		return ar_posix;
+	}
+
+	enum ar_type ret = ar_posix;
+
+	if (cmd_ctx.status == 0 && strstr(cmd_ctx.out.buf, "Free Software Foundation")) {
+		ret = ar_gcc;
+	}
+
+	run_cmd_ctx_destroy(&cmd_ctx);
+	return ret;
+}
+
+const char *
+ar_arguments(void)
+{
+	static enum ar_type ar_type;
+	static bool ar_type_initialized = false;
+
+	if (!ar_type_initialized) {
+		ar_type = compiler_detect_ar_type();
+		ar_type_initialized = true;
+	}
+
+	switch (ar_type) {
+	case ar_gcc:
+		return "csrD";
+	case ar_posix:
+		return "csr";
+	default:
+		UNREACHABLE_RETURN;
+	}
+}
