@@ -1,4 +1,9 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <string.h>
 
@@ -849,7 +854,7 @@ builtin_run(struct workspace *wk, bool have_rcvr, obj rcvr_id, uint32_t node_id,
 }
 
 bool
-analyze_function(struct workspace *wk, const struct func_impl_name *fi, uint32_t args_node, obj rcvr, obj *res)
+analyze_function(struct workspace *wk, const struct func_impl_name *fi, uint32_t args_node, obj rcvr, obj *res, bool *was_pure)
 {
 	struct analyze_function_opts old_opts = analyze_function_opts;
 	*res = 0;
@@ -877,6 +882,8 @@ analyze_function(struct workspace *wk, const struct func_impl_name *fi, uint32_t
 
 	analyze_function_opts = old_opts;
 
+	*was_pure = pure;
+
 	if (pure) {
 		return func_ret;
 	} else {
