@@ -1,4 +1,10 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-FileCopyrightText: dffdff2423 <dffdff2423@gmail.com>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <string.h>
 
@@ -8,6 +14,7 @@
 #include "error.h"
 #include "functions/common.h"
 #include "functions/environment.h"
+#include "functions/external_program.h"
 #include "functions/kernel.h"
 #include "functions/kernel/build_target.h"
 #include "functions/kernel/configure_file.h"
@@ -18,7 +25,6 @@
 #include "functions/kernel/subproject.h"
 #include "functions/modules.h"
 #include "functions/string.h"
-#include "guess.h"
 #include "lang/interpreter.h"
 #include "lang/serial.h"
 #include "log.h"
@@ -180,6 +186,12 @@ func_project(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 		}
 	}
 
+	if (wk->cur_project == 0) {
+		if (!prefix_dir_opts(wk)) {
+			return false;
+		}
+	}
+
 	if (wk->cur_project == 0 && akw[kw_subproject_dir].set) {
 		current_project(wk)->subprojects_dir = akw[kw_subproject_dir].val;
 	}
@@ -461,19 +473,6 @@ find_program_custom_dir_iter(struct workspace *wk, void *_ctx, obj val)
 	return ir_cont;
 }
 
-static void
-find_program_guess_version(struct workspace *wk, const char *path, obj *ver)
-{
-	*ver = 0;
-	struct run_cmd_ctx cmd_ctx = { 0 };
-	if (run_cmd_argv(&cmd_ctx, path, (char *const []){ (char *)path, "--version", 0 }, NULL, 0)
-	    && cmd_ctx.status == 0) {
-		guess_version(wk, cmd_ctx.out.buf, ver);
-	}
-
-	run_cmd_ctx_destroy(&cmd_ctx);
-}
-
 static bool
 find_program_check_override(struct workspace *wk, struct find_program_iter_ctx *ctx, obj prog)
 {
@@ -558,7 +557,7 @@ find_program(struct workspace *wk, struct find_program_iter_ctx *ctx, obj prog)
 {
 	const char *str;
 	obj ver = 0;
-	struct run_cmd_ctx cmd_ctx = { 0 };
+	bool guessed_ver = false;
 
 	if (!typecheck(wk, ctx->node, prog, tc_file | tc_string | tc_external_program)) {
 		return false;
@@ -652,12 +651,8 @@ find_program(struct workspace *wk, struct find_program_iter_ctx *ctx, obj prog)
 	return true;
 found:
 	if (ctx->version) {
-		if (run_cmd_argv(&cmd_ctx, path, (char *const []){ (char *)path, "--version", 0 }, NULL, 0)
-		    && cmd_ctx.status == 0) {
-			guess_version(wk, cmd_ctx.out.buf, &ver);
-		}
-
-		run_cmd_ctx_destroy(&cmd_ctx);
+		find_program_guess_version(wk, path, &ver);
+		guessed_ver = true;
 
 		if (!ver) {
 			return true; // no version to check against
@@ -675,6 +670,7 @@ found:
 	struct obj_external_program *ep = get_obj_external_program(wk, *ctx->res);
 	ep->found = true;
 	ep->full_path = make_str(wk, path);
+	ep->guessed_ver = guessed_ver;
 	ep->ver = ver;
 
 	ctx->found = true;
@@ -881,7 +877,7 @@ func_message(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 		return false;
 	}
 
-	log_plain(log_clr() ? "\033[34mmessage\033[0m: " : "message: ");
+	log_plain(log_clr() ? "\033[34mmessage\033[0m " : "message: ");
 	obj_array_foreach(wk, an[0].val, NULL, message_print_iter);
 	log_plain("\n");
 	*res = 0;
@@ -898,7 +894,7 @@ func_error(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 		return false;
 	}
 
-	log_plain(log_clr() ? "\033[31merror\033[0m: " : "error: ");
+	log_plain(log_clr() ? "\033[31merror\033[0m " : "error: ");
 	obj_array_foreach(wk, an[0].val, NULL, message_print_iter);
 	log_plain("\n");
 	*res = 0;
@@ -915,7 +911,7 @@ func_warning(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 		return false;
 	}
 
-	log_plain(log_clr() ? "\033[33mwarn:\033[0m " : "warn: ");
+	log_plain(log_clr() ? "\033[33mwarn\033[0m " : "warn: ");
 	obj_array_foreach(wk, an[0].val, NULL, message_print_iter);
 	log_plain("\n");
 	*res = 0;
@@ -926,7 +922,7 @@ func_warning(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 static bool
 func_run_command(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 {
-	struct args_norm an[] = { { ARG_TYPE_GLOB | tc_string | tc_file | tc_external_program }, ARG_TYPE_NULL };
+	struct args_norm an[] = { { ARG_TYPE_GLOB | tc_string | tc_file | tc_external_program | tc_compiler }, ARG_TYPE_NULL };
 	enum kwargs {
 		kw_check,
 		kw_env,
@@ -960,13 +956,22 @@ func_run_command(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 
 		obj_array_index(wk, an[0].val, 0, &arg0);
 
-		if (!find_program(wk, &find_program_ctx, arg0)) {
-			return false;
-		} else if (!find_program_ctx.found) {
-			interp_error(wk, an[0].node, "unable to find program %o", arg0);
-		}
+		if (get_obj_type(wk, arg0) == obj_compiler) {
+			obj cmd_arr = get_obj_compiler(wk, arg0)->cmd_arr;
 
-		obj_array_set(wk, an[0].val, 0, cmd_file);
+			obj tail;
+			obj_array_tail(wk, an[0].val, &tail);
+			obj_array_dup(wk, cmd_arr, &an[0].val);
+			obj_array_extend_nodup(wk, an[0].val, tail);
+		} else {
+			if (!find_program(wk, &find_program_ctx, arg0)) {
+				return false;
+			} else if (!find_program_ctx.found) {
+				interp_error(wk, an[0].node, "unable to find program %o", arg0);
+				return false;
+			}
+			obj_array_set(wk, an[0].val, 0, cmd_file);
+		}
 
 		obj args;
 		if (!arr_to_args(wk, arr_to_args_external_program, an[0].val, &args)) {
@@ -1127,7 +1132,7 @@ func_subdir(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 	}
 
 	path_push(wk, &new_cwd, "meson.build");
-	ret = wk->eval_project_file(wk, new_cwd.buf);
+	ret = wk->eval_project_file(wk, new_cwd.buf, false);
 
 ret:
 	current_project(wk)->cwd = old_cwd;
@@ -1630,7 +1635,7 @@ func_get_variable(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 		if (ao[0].set) {
 			*res = ao[0].val;
 		} else {
-			interp_error(wk, an[0].node, "undefined object");
+			interp_error(wk, an[0].node, "undefined object %o", an[0].val);
 			return false;
 		}
 	}
@@ -1785,8 +1790,8 @@ func_alias_target(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 	return true;
 }
 
-static bool
-func_range(struct workspace *wk, obj _, uint32_t args_node, obj *res)
+bool
+func_range_common(struct workspace *wk, uint32_t args_node, struct range_params *res)
 {
 	struct args_norm an[] = { { obj_number }, ARG_TYPE_NULL };
 	struct args_norm ao[] = { { obj_number }, { obj_number }, { obj_number }, ARG_TYPE_NULL };
@@ -1794,24 +1799,22 @@ func_range(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 		return false;
 	}
 
-	uint32_t start, stop, step;
-
 	int64_t n = get_obj_number(wk, an[0].val);
 	if (!rangecheck(wk, an[0].node, 0, UINT32_MAX, n)) {
 		return false;
 	}
-	start = n;
+	res->start = n;
 
 	if (ao[0].set) {
 		int64_t n = get_obj_number(wk, ao[0].val);
-		if (!rangecheck(wk, ao[0].node, start, UINT32_MAX, n)) {
+		if (!rangecheck(wk, ao[0].node, res->start, UINT32_MAX, n)) {
 			return false;
 		}
 
-		stop = n;
+		res->stop = n;
 	} else {
-		stop = start;
-		start = 0;
+		res->stop = res->start;
+		res->start = 0;
 	}
 
 	if (ao[1].set) {
@@ -1819,15 +1822,26 @@ func_range(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 		if (!rangecheck(wk, ao[1].node, 1, UINT32_MAX, n)) {
 			return false;
 		}
-		step = n;
+		res->step = n;
 	} else {
-		step = 1;
+		res->step = 1;
+	}
+
+	return true;
+}
+
+static bool
+func_range(struct workspace *wk, obj _, uint32_t args_node, obj *res)
+{
+	struct range_params params = { 0 };
+	if (!func_range_common(wk, args_node, &params)) {
+		return false;
 	}
 
 	make_obj(wk, res, obj_array);
 
 	uint32_t i;
-	for (i = start; i < stop; i += step) {
+	for (i = params.start; i < params.stop; i += params.step) {
 		obj num;
 		make_obj(wk, &num, obj_number);
 		set_obj_number(wk, num, i);
@@ -1954,7 +1968,7 @@ const struct func_impl_name impl_tbl_kernel[] =
 	{ "files", func_files, tc_array },
 	{ "find_program", func_find_program, tc_external_program },
 	{ "generator", func_generator, tc_generator },
-	{ "get_option", func_get_option, tc_string | tc_number | tc_bool | tc_feature_opt | tc_array, true },
+	{ "get_option", func_get_option, tc_string | tc_number | tc_bool | tc_feature_opt | tc_array, },
 	{ "get_variable", func_get_variable, tc_any, true },
 	{ "import", func_import, tc_module, true },
 	{ "include_directories", func_include_directories, tc_array },
@@ -1987,38 +2001,39 @@ const struct func_impl_name impl_tbl_kernel[] =
 	{ "warning", func_warning },
 	// non-standard muon extensions
 	{ "dbg", func_dbg, .extension = true },
-	{ "p", func_p, .extension = true },
+	{ "p", func_p, 0, true, .extension = true },
 	{ NULL, NULL },
 };
 
 const struct func_impl_name impl_tbl_kernel_internal[] = {
 	{ "assert", func_assert },
-	{ "configuration_data", func_configuration_data },
-	{ "disabler", func_disabler },
-	{ "environment", func_environment },
+	{ "configuration_data", func_configuration_data, tc_configuration_data },
+	{ "disabler", func_disabler, tc_disabler },
+	{ "environment", func_environment, tc_environment },
 	{ "error", func_error },
-	{ "files", func_files },
-	{ "find_program", func_find_program },
-	{ "get_variable", func_get_variable },
-	{ "import", func_import },
-	{ "is_disabler", func_is_disabler },
-	{ "is_variable", func_is_variable },
-	{ "join_paths", func_join_paths },
+	{ "files", func_files, tc_array },
+	{ "find_program", func_find_program, tc_external_program },
+	{ "get_variable", func_get_variable, tc_any, true },
+	{ "import", func_import, tc_module, true },
+	{ "is_disabler", func_is_disabler, tc_bool, true },
+	{ "is_variable", func_is_variable, tc_bool, true },
+	{ "join_paths", func_join_paths, tc_string, true },
 	{ "message", func_message },
-	{ "range", func_range },
-	{ "run_command", func_run_command, .fuzz_unsafe = true },
-	{ "set_variable", func_set_variable },
+	{ "range", func_range, tc_array, true },
+	{ "run_command", func_run_command, tc_run_result, .fuzz_unsafe = true },
+	{ "set_variable", func_set_variable, 0, true },
+	{ "unset_variable", func_unset_variable, 0, true },
 	{ "warning", func_warning },
 	// non-standard muon extensions
 	{ "dbg", func_dbg },
 	{ "p", func_p },
-	{ "serial_load", func_serial_load },
+	{ "serial_load", func_serial_load, tc_any },
 	{ "serial_dump", func_serial_dump, .fuzz_unsafe = true },
 	{ NULL, NULL },
 };
 
 const struct func_impl_name impl_tbl_kernel_opts[] = {
-	{ "option", func_option, 0, true  },
+	{ "option", func_option, 0, true },
 	// non-standard muon extensions
 	{ "dbg", func_dbg },
 	{ "p", func_p },
