@@ -1,4 +1,11 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-FileCopyrightText: illiliti <illiliti@dimension.sh>
+ * SPDX-FileCopyrightText: illiliti <illiliti@thunix.net>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <assert.h>
 #include <string.h>
@@ -11,6 +18,7 @@
 #include "functions/generator.h"
 #include "functions/kernel/build_target.h"
 #include "functions/kernel/dependency.h"
+#include "functions/machine.h"
 #include "install.h"
 #include "lang/interpreter.h"
 #include "log.h"
@@ -55,9 +63,12 @@ enum build_target_kwargs {
 
 	/* lang args */
 	bt_kw_c_pch, // TODO
+	bt_kw_cpp_pch, // TODO
 	bt_kw_c_args,
 	bt_kw_cpp_args,
 	bt_kw_objc_args,
+	bt_kw_masm_args, // TODO
+	bt_kw_nasm_args, // TODO
 	bt_kw_link_args,
 
 	bt_kwargs_count,
@@ -80,14 +91,44 @@ determine_linker_iter(struct workspace *wk, void *_ctx, obj val)
 	return ir_cont;
 }
 
+static enum iteration_result
+determine_linker_from_objects_iter(struct workspace *wk, void *_ctx, obj val)
+{
+	struct obj_build_target *tgt = _ctx;
+
+	enum compiler_language fl;
+
+	const struct str *o = get_str(wk, *get_obj_file(wk, val));
+	if (!str_endswith(o, &WKSTR(".o"))) {
+		return ir_cont;
+	}
+
+	SBUF(path);
+	sbuf_pushs(wk, &path, o->s);
+	path.len -= 2;
+	path.buf[path.len] = 0;
+
+	if (!filename_to_compiler_language(path.buf, &fl)) {
+		/* LOG_E("unable to determine language for '%s'", get_cstr(wk, src->dat.file)); */
+		return ir_cont;
+	}
+
+	tgt->dep_internal.link_language = coalesce_link_languages(tgt->dep_internal.link_language, fl);
+
+	return ir_cont;
+}
+
 static bool
 build_tgt_determine_linker(struct workspace *wk, uint32_t err_node, struct obj_build_target *tgt)
 {
-
 	if (!obj_array_foreach(wk, tgt->src, tgt, determine_linker_iter)) {
 		return ir_err;
 	}
 
+	if (!obj_array_foreach(wk, tgt->objects, tgt, determine_linker_from_objects_iter)) {
+		return ir_err;
+	}
+
 	if (!tgt->dep_internal.link_language) {
 		enum compiler_language clink_langs[] = {
 			compiler_language_c,
@@ -261,6 +302,27 @@ setup_soname(struct workspace *wk, struct obj_build_target *tgt, const char *pla
 	tgt->soname = make_strf(wk, "%s%s", plain_name, have_soversion ? soversion : "");
 }
 
+static void
+setup_dllname(struct workspace *wk, struct obj_build_target *tgt, const char *plain_name, obj dllver, obj ver)
+{
+	if (dllver) {
+		tgt->soname = make_strf(wk, "%s-%s.dll", plain_name, get_cstr(wk, dllver));
+		return;
+	} else if (ver) {
+		char buf[BUF_SIZE_1k];
+
+		strncpy(buf, get_cstr(wk, ver), sizeof(buf) - 1);
+		char *tmp = strchr(buf, '.');
+		if (tmp) {
+			*tmp = '\0';
+			tgt->soname = make_strf(wk, "%s-%s.dll", plain_name, buf);
+			return;
+		}
+	}
+
+	tgt->soname = make_strf(wk, "%s.dll", plain_name);
+}
+
 static bool
 setup_shared_object_symlinks(struct workspace *wk, struct obj_build_target *tgt,
 	const char *plain_name, obj *plain_name_install, obj *soname_install)
@@ -299,26 +361,58 @@ static bool
 determine_target_build_name(struct workspace *wk, struct obj_build_target *tgt, obj sover, obj ver,
 	obj name_pre, obj name_suff, char plain_name[BUF_SIZE_2k])
 {
+	char ver_dll[BUF_SIZE_1k];
 	const char *pref, *suff, *ver_suff = NULL;
-	uint32_t i;
+	enum machine_system sys = machine_system();
+
+	*ver_dll = '\0';
 
 	switch (tgt->type) {
 	case tgt_executable:
 		pref = "";
-		suff = NULL;
+		if (sys == machine_system_windows || sys == machine_system_cygwin) {
+			suff = "exe";
+		} else {
+			suff = NULL;
+		}
 		break;
 	case tgt_static_library:
-		pref = "lib";
+		if (sys == machine_system_cygwin) {
+			pref = "cyg";
+		} else {
+			pref = "lib";
+		}
 		suff = "a";
 		break;
 	case tgt_shared_module:
 	case tgt_dynamic_library:
-		pref = "lib";
-		suff = "so";
-		if (ver) {
-			ver_suff = get_cstr(wk, ver);
-		} else if (sover) {
-			ver_suff = get_cstr(wk, sover);
+		if (sys == machine_system_cygwin) {
+			pref = "cyg";
+		} else {
+			pref = "lib";
+		}
+		if (sys == machine_system_windows || sys == machine_system_cygwin) {
+			suff = "dll";
+			if (sover) {
+				strncpy(ver_dll, get_cstr(wk, sover), sizeof(ver_dll) - 1);
+			} else if (ver) {
+				strncpy(ver_dll, get_cstr(wk, ver), sizeof(ver_dll) - 1);
+				char *ver_tmp = strchr(ver_dll, '.');
+				if (ver_tmp) {
+					*ver_tmp = '\0';
+				} else {
+					*ver_dll = '\0';
+				}
+			}
+		} else if (sys == machine_system_darwin) {
+			suff = "dylib";
+		} else {
+			suff = "so";
+			if (ver) {
+				ver_suff = get_cstr(wk, ver);
+			} else if (sover) {
+				ver_suff = get_cstr(wk, sover);
+			}
 		}
 		break;
 	default:
@@ -334,12 +428,14 @@ determine_target_build_name(struct workspace *wk, struct obj_build_target *tgt,
 		suff = get_cstr(wk, name_suff);
 	}
 
-	i = snprintf(plain_name, BUF_SIZE_2k, "%s%s", pref, get_cstr(wk, tgt->name));
-	if (suff) {
-		snprintf(&plain_name[i], BUF_SIZE_2k, ".%s", suff);
+	if (sys == machine_system_windows || sys == machine_system_cygwin) {
+		snprintf(plain_name, BUF_SIZE_2k, "%s%s", pref, get_cstr(wk, tgt->name));
+		tgt->build_name = make_strf(wk, "%s%s%s%s%s", plain_name, *ver_dll ? "-" : "", *ver_dll ? ver_dll : "", suff ? "." : "", suff ? suff : "");
+	} else {
+		snprintf(plain_name, BUF_SIZE_2k, "%s%s%s%s", pref, get_cstr(wk, tgt->name), suff ? "." : "", suff ? suff : "");
+		tgt->build_name = make_strf(wk, "%s%s%s", plain_name, ver_suff ? "." : "", ver_suff ? ver_suff : "");
 	}
 
-	tgt->build_name = make_strf(wk, "%s%s%s", plain_name, ver_suff ? "." : "", ver_suff ? ver_suff : "");
 	return true;
 }
 
@@ -570,7 +666,8 @@ create_target(struct workspace *wk, struct args_norm *an, struct args_kw *akw,
 		} lang_args[] = {
 			{ bt_kw_c_args, compiler_language_c },
 			{ bt_kw_cpp_args, compiler_language_cpp },
-			/* { bt_kw_objc_args, compiler_language_objc }, */
+			{ bt_kw_objc_args, compiler_language_objc },
+			{ bt_kw_nasm_args, compiler_language_nasm },
 		};
 
 		uint32_t i;
@@ -585,13 +682,18 @@ create_target(struct workspace *wk, struct args_norm *an, struct args_kw *akw,
 
 	// soname handling
 	if (type & (tgt_dynamic_library | tgt_shared_module)) {
+		enum machine_system sys = machine_system();
 
-		setup_soname(wk, tgt, plain_name, sover, akw[bt_kw_version].val);
+		if (sys == machine_system_windows || sys == machine_system_cygwin) {
+			setup_dllname(wk, tgt, plain_name, sover, akw[bt_kw_version].val);
+		} else {
+			setup_soname(wk, tgt, plain_name, sover, akw[bt_kw_version].val);
 
-		if (type == tgt_dynamic_library) {
-			if (!setup_shared_object_symlinks(wk, tgt, plain_name,
-				&plain_name_install, &soname_install)) {
-				return false;
+			if (type == tgt_dynamic_library) {
+				if (!setup_shared_object_symlinks(wk, tgt, plain_name,
+					&plain_name_install, &soname_install)) {
+					return false;
+				}
 			}
 		}
 	}
@@ -617,11 +719,20 @@ create_target(struct workspace *wk, struct args_norm *an, struct args_kw *akw,
 			case tgt_executable:
 				get_option_value(wk, current_project(wk), "bindir", &install_dir);
 				break;
-			case tgt_dynamic_library:
 			case tgt_static_library:
-			case tgt_shared_module:
 				get_option_value(wk, current_project(wk), "libdir", &install_dir);
 				break;
+			case tgt_dynamic_library:
+			case tgt_shared_module: {
+				enum machine_system sys = machine_system();
+
+				if (sys == machine_system_windows || sys == machine_system_cygwin) {
+					get_option_value(wk, current_project(wk), "bindir", &install_dir);
+				} else {
+					get_option_value(wk, current_project(wk), "libdir", &install_dir);
+				}
+				break;
+			}
 			default:
 				assert(false && "unreachable");
 				break;
@@ -752,9 +863,12 @@ tgt_common(struct workspace *wk, uint32_t args_node, obj *res, enum tgt_type typ
 		[bt_kw_override_options] = { "override_options", ARG_TYPE_ARRAY_OF | obj_string },
 		/* lang args */
 		[bt_kw_c_pch] = { "c_pch", tc_string | tc_file, },
+		[bt_kw_cpp_pch] = { "cpp_pch", tc_string | tc_file, },
 		[bt_kw_c_args] = { "c_args", ARG_TYPE_ARRAY_OF | obj_string },
 		[bt_kw_cpp_args] = { "cpp_args", ARG_TYPE_ARRAY_OF | obj_string },
 		[bt_kw_objc_args] = { "objc_args", ARG_TYPE_ARRAY_OF | obj_string },
+		[bt_kw_nasm_args] = { "nasm_args", ARG_TYPE_ARRAY_OF | obj_string },
+		[bt_kw_masm_args] = { "masm_args", ARG_TYPE_ARRAY_OF | obj_string },
 		[bt_kw_link_args] = { "link_args", ARG_TYPE_ARRAY_OF | obj_string },
 		0
 	};
