@@ -1,4 +1,12 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-FileCopyrightText: dffdff2423 <dffdff2423@gmail.com>
+ * SPDX-FileCopyrightText: Eli Schwartz <eschwartz@archlinux.org>
+ * SPDX-FileCopyrightText: illiliti <illiliti@thunix.net>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -162,7 +170,7 @@ format_cmd_arg_cb(struct workspace *wk, uint32_t node, void *_ctx, const struct
 	case key_depfile:
 		/* @DEPFILE@: the full path to the dependency file passed to
 		 * depfile */
-		str_relative_to_build_root(wk, ctx, get_cstr(wk, ctx->opts->depfile), elem);
+		str_relative_to_build_root(wk, ctx, get_file_path(wk, ctx->opts->depfile), elem);
 		return format_cb_found;
 	case key_source_root:
 		/* @SOURCE_ROOT@: the path to the root of the source tree.
@@ -300,6 +308,14 @@ custom_target_cmd_fmt_iter(struct workspace *wk, void *_ctx, obj val)
 		}
 		goto cont;
 	}
+	case obj_compiler: {
+		obj cmd_array = get_obj_compiler(wk, val)->cmd_arr;
+
+		if (!obj_array_foreach(wk, cmd_array, ctx, custom_target_cmd_fmt_iter)) {
+			return ir_err;
+		}
+		goto cont;
+	}
 	default:
 		interp_error(wk, ctx->opts->err_node, "unable to coerce %o to string", val);
 		return ir_err;
@@ -511,13 +527,38 @@ make_custom_target(struct workspace *wk,
 		output = 0;
 	}
 
+	obj depfile = 0;
+	if (opts->depfile_orig) {
+		obj raw_depfiles;
+		if (!coerce_output_files(wk, 0, opts->depfile_orig, opts->output_dir, &raw_depfiles)) {
+			return false;
+		}
+
+		if (!obj_array_flatten_one(wk, raw_depfiles, &depfile)) {
+			UNREACHABLE;
+		}
+
+		struct custom_target_cmd_fmt_ctx ctx = {
+			.opts = &(struct process_custom_target_commandline_opts) {
+				.input = input,
+			},
+		};
+
+		obj depfile_formatted;
+		if (!string_format(wk, 0, *get_obj_file(wk, depfile), &depfile_formatted, &ctx, format_cmd_output_cb)) {
+			return ir_err;
+		}
+
+		*get_obj_file(wk, depfile) = depfile_formatted;
+	}
+
 	struct process_custom_target_commandline_opts cmdline_opts = {
 		.err_node   = opts->command_node,
 		.relativize = true,
 		.name       = opts->name,
 		.input      = input,
 		.output     = output,
-		.depfile    = opts->depfile_orig,
+		.depfile    = depfile,
 		.build_dir  = opts->build_dir,
 		.extra_args = opts->extra_args,
 		.extra_args_valid = opts->extra_args_valid,
@@ -542,6 +583,7 @@ make_custom_target(struct workspace *wk,
 	tgt->args = args;
 	tgt->input = input;
 	tgt->output = output;
+	tgt->depfile = depfile;
 	tgt->depends = cmdline_opts.depends;
 	return true;
 }
@@ -549,7 +591,7 @@ make_custom_target(struct workspace *wk,
 bool
 func_custom_target(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 {
-	struct args_norm an[] = { { obj_string }, ARG_TYPE_NULL };
+	struct args_norm ao[] = { { obj_string }, ARG_TYPE_NULL };
 	enum kwargs {
 		kw_input,
 		kw_output,
@@ -590,12 +632,26 @@ func_custom_target(struct workspace *wk, obj _, uint32_t args_node, obj *res)
 		0
 	};
 
-	if (!interp_args(wk, args_node, an, NULL, akw)) {
+	if (!interp_args(wk, args_node, NULL, ao, akw)) {
 		return false;
 	}
 
+	obj name;
+	if (ao[0].set) {
+		name = ao[0].val;
+	} else {
+		if (!get_obj_array(wk, akw[kw_output].val)->len) {
+			interp_error(wk, akw[kw_output].node, "output cannot be empty");
+			return false;
+		}
+
+		obj v;
+		obj_array_index(wk, akw[kw_output].val, 0, &v);
+		name = v;
+	}
+
 	struct make_custom_target_opts opts = {
-		.name         = an[0].val,
+		.name         = name,
 		.input_node   = akw[kw_input].node,
 		.output_node  = akw[kw_output].node,
 		.command_node = akw[kw_command].node,
