@@ -1,4 +1,10 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-FileCopyrightText: illiliti <illiliti@dimension.sh>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <assert.h>
 #include <string.h>
@@ -358,6 +364,9 @@ module_pkgconf_process_libs_iter(struct workspace *wk, void *_ctx, obj val)
 			}
 			break;
 		}
+		case dependency_type_appleframeworks:
+		       // TODO: actually add correct -framework arguments
+		       break;
 		}
 		break;
 	}
