@@ -1,4 +1,10 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-FileCopyrightText: Ailin Nemui <ailin@d5421s.localdomain>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <inttypes.h>
 #include <stdlib.h>
@@ -8,6 +14,7 @@
 #include "error.h"
 #include "formats/ini.h"
 #include "lang/fmt.h"
+#include "lang/string.h"
 #include "log.h"
 #include "platform/mem.h"
 
@@ -17,10 +24,12 @@ struct arg_elem {
 
 struct fmt_ctx {
 	struct ast *ast;
-	FILE *out;
+	struct sbuf *out_buf;
 	uint32_t indent, col, enclosed;
 	bool force_ml;
+	bool trailing_comment;
 
+	/* options */
 	bool space_array, kwa_ml, wide_colon, no_single_comma_function;
 	uint32_t max_line_len;
 	const char *indent_by;
@@ -46,6 +55,12 @@ typedef uint32_t ((*fmt_func)(struct fmt_ctx *ctx, const struct fmt_stack *pfst,
 static uint32_t fmt_node(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id);
 static uint32_t fmt_chain(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id);
 
+static const char *
+get_comment(struct fmt_ctx *ctx, struct node *n, uint32_t i)
+{
+	return *(const char **)darr_get(&ctx->ast->comments, n->comments.start + i);
+}
+
 static bool
 fmt_str_startwith(struct fmt_ctx *ctx, uint32_t n_id, const char *pre)
 {
@@ -128,7 +143,7 @@ fmt_write(struct fmt_ctx *ctx, const struct fmt_stack *pfst, char c)
 	}
 
 	if (pfst->write) {
-		fputc(c, ctx->out);
+		sbuf_push(NULL, ctx->out_buf, c);
 		ctx->col += len;
 	}
 	return len;
@@ -141,7 +156,7 @@ fmt_writeml(struct fmt_ctx *ctx, const struct fmt_stack *pfst, const char *s)
 	for (; *s; ++s) {
 		if (*s == '\n') {
 			if (pfst->write) {
-				fputc('\n', ctx->out);
+				sbuf_push(NULL, ctx->out_buf, '\n');
 				ctx->col = 0;
 			}
 
@@ -168,8 +183,12 @@ static void
 fmt_newline(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t next)
 {
 	uint32_t i;
+	if (pfst->ml) {
+		ctx->trailing_comment = false;
+	}
+
 	if (pfst->write && pfst->ml) {
-		fputc('\n', ctx->out);
+		sbuf_push(NULL, ctx->out_buf, '\n');
 		ctx->col = 0;
 
 		if (next != 0) {
@@ -178,10 +197,9 @@ fmt_newline(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t next)
 				n = get_node(ctx->ast, n->l);
 			}
 
-			if (n->type == node_empty_line && !n->comment) {
+			if (n->type == node_empty_line && !n->comments.len) {
 				return;
 			}
-
 		}
 
 		for (i = 0; i < ctx->indent; ++i) {
@@ -234,7 +252,7 @@ fmt_end_block(struct fmt_ctx *ctx)
 	--ctx->indent;
 }
 
-__attribute__ ((format(printf, 3, 4)))
+MUON_ATTR_FORMAT(printf, 3, 4)
 static uint32_t
 fmt_writef(struct fmt_ctx *ctx, const struct fmt_stack *pfst, const char *fmt, ...)
 {
@@ -253,25 +271,62 @@ fmt_writef(struct fmt_ctx *ctx, const struct fmt_stack *pfst, const char *fmt, .
 	return len;
 }
 
+static uint32_t
+fmt_comments(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id, bool allow_leading_space)
+{
+	struct node *n = get_node(ctx->ast, n_id);
+	uint32_t len = 0;
+
+	if (!n->comments.len) {
+		return 0;
+	}
+
+	bool leading_space = allow_leading_space && n->type != node_empty_line;
+
+	ctx->force_ml = true;
+
+	uint32_t i;
+	for (i = 0; i < n->comments.len; ++i) {
+		len += fmt_writef(ctx, pfst, "%s#%s",
+			leading_space ? " " : "",
+			get_comment(ctx, n, i));
+
+		if (i < n->comments.len - 1) { // && !trailing_line) {
+			fmt_newline_force(ctx, pfst, n_id);
+		} else {
+			if (pfst->write) {
+				ctx->trailing_comment = true;
+			}
+		}
+
+		leading_space = false;
+	}
+
+	return len;
+}
+
+static uint32_t
+fmt_check_trailing_comment_or_space(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id)
+{
+	if (ctx->trailing_comment) {
+		fmt_newline_force(ctx, pfst, n_id);
+		return 0;
+	} else {
+		fmt_write(ctx, pfst, ' ');
+		return 1;
+	}
+}
+
 static uint32_t
 fmt_tail(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id)
 {
 	uint32_t len = 0;
-	struct node *n = get_node(ctx->ast, n_id);
 
 	if (pfst->node_sep) {
 		len += fmt_writes(ctx, pfst, pfst->node_sep);
 	}
 
-	if (n->comment) {
-		len += fmt_writef(ctx, pfst, "%s#%s",
-			n->type == node_empty_line ? "" : " ",
-			n->comment);
-		if (ctx->enclosed) {
-			ctx->force_ml = true;
-		}
-	}
-
+	len += fmt_comments(ctx, pfst, n_id, true);
 	return len;
 }
 
@@ -444,14 +499,15 @@ static uint32_t
 fmt_arg_container(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id)
 {
 	struct fmt_stack fst = *fmt_setup_fst(pfst);
-	struct node *p = get_node(ctx->ast, fst.parent);
+	/* struct node *p = get_node(ctx->ast, fst.parent); */
 
 	uint32_t len = 0;
 	len += fmt_write(ctx, &fst, fst.arg_container[0]);
-	if (p->comment) {
-		len += fmt_writef(ctx, pfst, " #%s", p->comment);
-		ctx->force_ml = true;
-	}
+	len += fmt_comments(ctx, pfst, fst.parent, true);
+	/* if (p->comment) { */
+	/* 	len += fmt_writef(ctx, pfst, " #%s", p->comment); */
+	/* 	ctx->force_ml = true; */
+	/* } */
 
 	len += fmt_args(ctx, &fst, n_id);
 	len += fmt_write(ctx, &fst, fst.arg_container[1]);
@@ -581,6 +637,7 @@ fmt_parens(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id)
 		fmt_newline(ctx, &fst, n->l);
 	}
 
+	len += fmt_comments(ctx, &fst, n_id, false);
 	len += fmt_node(ctx, &fst, n->l);
 
 	if (fst.ml) {
@@ -608,11 +665,18 @@ fmt_if(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id, bool fi
 		fmt_writes(ctx, &fst, "else");
 	}
 
-	fmt_begin_block(ctx);
-	fmt_newline_force(ctx, &fst, n->r);
-	fmt_node(ctx, &fst, n->r);
-	fmt_end_block(ctx);
-	fmt_newline_force(ctx, &fst, n->c);
+	struct node *block = get_node(ctx->ast, n->r);
+	if (block->type == node_empty) {
+		fmt_comments(ctx, pfst, n->r, true);
+		fmt_newline_force(ctx, &fst, n->r);
+	} else {
+		fmt_begin_block(ctx);
+		fmt_newline_force(ctx, &fst, n->r);
+		fmt_node(ctx, &fst, n->r);
+		fmt_end_block(ctx);
+		fmt_newline_force(ctx, &fst, n->c);
+	}
+
 	if (n->chflg & node_child_c) {
 		fmt_if(ctx, &fst, n->c, false);
 	}
@@ -683,7 +747,13 @@ fmt_node(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id)
 	uint32_t len = 0;
 	struct node *n = get_node(ctx->ast, n_id);
 
-	/* L("formatting %s", node_to_s(n)); */
+	if (ctx->trailing_comment && n->type != node_empty_line) {
+		fmt_newline_force(ctx, pfst, n_id);
+	}
+
+	/* if (pfst->write) { */
+	/* 	L("formatting %s", node_to_s(n)); */
+	/* } */
 
 	switch (n->type) {
 	/* literals */
@@ -740,13 +810,17 @@ fmt_node(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id)
 		fmt_writes(ctx, &fst, " : ");
 		fmt_check(ctx, &fst, fmt_node_wrapped, n->r);
 
-		fmt_begin_block(ctx);
-		fmt_newline_force(ctx, &fst, n->c);
-
-		fmt_node(ctx, &fst, n->c);
-
-		fmt_end_block(ctx);
-		fmt_newline_force(ctx, &fst, 0);
+		struct node *block = get_node(ctx->ast, n->c);
+		if (block->type == node_empty) {
+			fmt_comments(ctx, pfst, n->c, true);
+			fmt_newline_force(ctx, &fst, n->r);
+		} else {
+			fmt_begin_block(ctx);
+			fmt_newline_force(ctx, &fst, n->c);
+			fmt_node(ctx, &fst, n->c);
+			fmt_end_block(ctx);
+			fmt_newline_force(ctx, &fst, 0);
+		}
 
 		fmt_writes(ctx, &fst, "endforeach");
 		break;
@@ -773,12 +847,14 @@ fmt_node(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id)
 	/* assignment */
 	case node_assignment:
 		len += fmt_node(ctx, &fst, n->l);
-		len += fmt_writes(ctx, &fst, " = ");
+		len += fmt_check_trailing_comment_or_space(ctx, &fst, n->l);
+		len += fmt_writes(ctx, &fst, "= ");
 		len += fmt_node_wrapped(ctx, &fst, n->r);
 		break;
 	case node_plusassign:
 		len += fmt_node(ctx, &fst, n->l);
-		len += fmt_writes(ctx, &fst, " += ");
+		len += fmt_check_trailing_comment_or_space(ctx, &fst, n->l);
+		len += fmt_writes(ctx, &fst, "+= ");
 		len += fmt_node_wrapped(ctx, &fst, n->r);
 		break;
 
@@ -806,15 +882,18 @@ fmt_node(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id)
 		}[n->subtype];
 
 		len += fmt_node(ctx, &fst, n->l);
-		len += fmt_writef(ctx, &fst, " %s ", kw);
+		len += fmt_check_trailing_comment_or_space(ctx, &fst, n->l);
+		len += fmt_writef(ctx, &fst, "%s ", kw);
 		len += fmt_node(ctx, &fst, n->r);
 		break;
 	}
 	case node_ternary:
 		len += fmt_node(ctx, &fst, n->l);
-		len += fmt_writef(ctx, &fst, " ? ");
+		len += fmt_check_trailing_comment_or_space(ctx, &fst, n->l);
+		len += fmt_writef(ctx, &fst, "? ");
 		len += fmt_node(ctx, &fst, n->r);
-		len += fmt_writef(ctx, &fst, " : ");
+		len += fmt_check_trailing_comment_or_space(ctx, &fst, n->l);
+		len += fmt_writef(ctx, &fst, ": ");
 		len += fmt_node(ctx, &fst, n->c);
 		break;
 
@@ -825,7 +904,7 @@ fmt_node(struct fmt_ctx *ctx, const struct fmt_stack *pfst, uint32_t n_id)
 		break;
 	case node_arithmetic:
 		len += fmt_node(ctx, &fst, n->l);
-		len += fmt_write(ctx, &fst, ' ');
+		len += fmt_check_trailing_comment_or_space(ctx, &fst, n->l);
 		len += fmt_write(ctx, &fst, "+-%*/"[n->subtype]);
 		len += fmt_breaking_space(ctx, &fst, n->r);
 		len += fmt_node(ctx, &fst, n->r);
@@ -874,7 +953,8 @@ fmt_cfg_parse_cb(void *_ctx, struct source *src, const char *sect,
 		{ "max_line_len", type_uint, offsetof(struct fmt_ctx, max_line_len) },
 		{ "indent_by", type_str, offsetof(struct fmt_ctx, indent_by) },
 		{ "space_array", type_bool, offsetof(struct fmt_ctx, space_array) },
-		{ "kwa_ml", type_bool, offsetof(struct fmt_ctx, kwa_ml) },
+		{ "kwargs_force_multiline", type_bool, offsetof(struct fmt_ctx, kwa_ml) },
+		{ "kwa_ml", type_bool, offsetof(struct fmt_ctx, kwa_ml) }, // kept for backwards compat
 		{ "wide_colon", type_bool, offsetof(struct fmt_ctx, wide_colon) },
 		{ "no_single_comma_function", type_bool, offsetof(struct fmt_ctx, no_single_comma_function) },
 		0
@@ -950,12 +1030,15 @@ fmt_cfg_parse_cb(void *_ctx, struct source *src, const char *sect,
 }
 
 bool
-fmt(struct ast *ast, FILE *out, const char *cfg_path)
+fmt(struct source *src, FILE *out, const char *cfg_path, bool check_only)
 {
 	bool ret = false;
+	struct ast ast = { 0 };
+	struct source_data sdata = { 0 };
+	struct sbuf out_buf;
 	struct fmt_ctx ctx = {
-		.ast = ast,
-		.out = out,
+		.ast = &ast,
+		.out_buf = &out_buf,
 		.max_line_len = 80,
 		.indent_by = "    ",
 		.space_array = false,
@@ -963,30 +1046,51 @@ fmt(struct ast *ast, FILE *out, const char *cfg_path)
 		.wide_colon = false,
 		.no_single_comma_function = false,
 	};
-
-	char *buf = NULL;
-	struct source cfg_src = { 0 };
-	if (cfg_path) {
-		if (!fs_read_entire_file(cfg_path, &cfg_src)) {
-			goto ret;
-		} else if (!ini_parse(cfg_path, &cfg_src, &buf, fmt_cfg_parse_cb, &ctx)) {
-			goto ret;
-		}
-	}
-
 	struct fmt_stack fst = {
 		.write = true,
 	};
 
-	fmt_node(&ctx, &fst, ast->root);
+	if (check_only) {
+		sbuf_init(&out_buf, NULL, 0, sbuf_flag_overflow_alloc);
+	} else {
+		out_buf.flags = sbuf_flag_write;
+		out_buf.buf = (void *)out;
+	}
+
+	char *cfg_buf = NULL;
+	struct source cfg_src = { 0 };
+	if (cfg_path) {
+		if (!fs_read_entire_file(cfg_path, &cfg_src)) {
+			goto ret;
+		} else if (!ini_parse(cfg_path, &cfg_src, &cfg_buf, fmt_cfg_parse_cb, &ctx)) {
+			goto ret;
+		}
+	}
+
+	if (!parser_parse(NULL, &ast, &sdata, src, pm_keep_formatting | pm_ignore_statement_with_no_effect)) {
+		goto ret;
+	}
+
+	fmt_node(&ctx, &fst, ast.root);
 	assert(!ctx.indent);
 	fmt_newline_force(&ctx, &fst, 0);
 
+	if (check_only) {
+		if (src->len != out_buf.len) {
+			goto ret;
+		} else if (memcmp(src->src, out_buf.buf, src->len)) {
+			goto ret;
+		}
+	}
+
 	ret = true;
 ret:
-	if (buf) {
-		z_free(buf);
+	if (cfg_buf) {
+		z_free(cfg_buf);
 	}
+	sbuf_destroy(&out_buf);
 	fs_source_destroy(&cfg_src);
+	ast_destroy(&ast);
+	source_data_destroy(&sdata);
 	return ret;
 }
