@@ -1,4 +1,10 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-FileCopyrightText: Simon Zeni <simon@bl4ckb0ne.ca>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <assert.h>
 #include <errno.h>
@@ -87,7 +93,7 @@ tok_type_to_s(enum token_type type)
 	return "";
 }
 
-__attribute__ ((format(printf, 2, 3)))
+MUON_ATTR_FORMAT(printf, 2, 3)
 static void
 lex_error(struct lexer *l, const char *fmt, ...)
 {
@@ -707,7 +713,6 @@ lexer_tokenize_one(struct lexer *lexer)
 	} else {
 		switch (lexer->src[lexer->i]) {
 		case '\n':
-
 			if (lexer->enclosing.paren || lexer->enclosing.bracket
 			    || lexer->enclosing.curl) {
 				if (lexer->mode & lexer_mode_format) {
