@@ -1,4 +1,11 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-FileCopyrightText: Simon Zeni <simon@bl4ckb0ne.ca>
+ * SPDX-FileCopyrightText: illiliti <illiliti@dimension.sh>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <inttypes.h>
 #include <stdarg.h>
@@ -258,6 +265,7 @@ make_obj(struct workspace *wk, obj *id, enum obj_type type)
 			mem += bucket_array_size(&wk->obj_aos[i]);
 		}
 #define MB(b) ((double)b / 1048576.0)
+		TracyCPlot("objects", wk->objs.len);
 		TracyCPlot("object memory (mb)", MB(mem));
 		TracyCPlot("string memory (mb)", MB(bucket_array_size(&wk->chrs)));
 #undef MB
@@ -548,6 +556,16 @@ obj_array_push(struct workspace *wk, obj arr, obj child)
 	++a->len;
 }
 
+void
+obj_array_prepend(struct workspace *wk, obj *arr, obj val)
+{
+	obj prepend;
+	make_obj(wk, &prepend, obj_array);
+	obj_array_push(wk, prepend, val);
+	obj_array_extend_nodup(wk, prepend, *arr);
+	*arr = prepend;
+}
+
 struct obj_array_index_of_iter_ctx {
 	obj l;
 	bool res;
@@ -727,6 +745,22 @@ obj_array_join(struct workspace *wk, bool flat, obj arr, obj join, obj *res)
 	}
 }
 
+void
+obj_array_tail(struct workspace *wk, obj arr, obj *res)
+{
+	const struct obj_array *a = get_obj_array(wk, arr);
+
+	if (a->have_next) {
+		struct obj_array *n = get_obj_array(wk, a->next);
+		n->tail = a->tail;
+		n->len = a->len;
+		*res = a->next;
+	} else {
+		// the tail of a zero or single element array is an empty array
+		make_obj(wk, res, obj_array);
+	}
+}
+
 void
 obj_array_set(struct workspace *wk, obj arr, int64_t i, obj v)
 {
@@ -1560,24 +1594,22 @@ obj_to_s(struct workspace *wk, obj o, struct sbuf *sb)
 	}
 }
 
+#define FMT_PARTIAL(arg) \
+	if (arg_width.have && arg_prec.have) { \
+		sbuf_pushf(wk, sb, fmt_buf, arg_width.val, arg_prec.val, arg); \
+	} else if (arg_width.have) { \
+		sbuf_pushf(wk, sb, fmt_buf, arg_width.val, arg); \
+	} else if (arg_prec.have) { \
+		sbuf_pushf(wk, sb, fmt_buf, arg_prec.val, arg); \
+	} else { \
+		sbuf_pushf(wk, sb, fmt_buf, arg); \
+	}
+
 bool
 obj_vasprintf(struct workspace *wk, struct sbuf *sb, const char *fmt, va_list ap)
 {
 	const char *fmt_start;
-	obj obj;
-	bool got_object, quote_string;
-
-	union {
-		int _int;
-		long _lint;
-		long long _llint;
-		unsigned int _uint;
-		unsigned long _luint;
-		unsigned long long _lluint;
-		double _double;
-		char *_charp;
-		void *_voidp;
-	} arg = { 0 };
+	bool quote_string;
 
 	struct {
 		int val;
@@ -1589,7 +1621,6 @@ obj_vasprintf(struct workspace *wk, struct sbuf *sb, const char *fmt, va_list ap
 			arg_width.have = false;
 			arg_prec.have = false;
 
-			got_object = false;
 			quote_string = true;
 			fmt_start = fmt;
 			++fmt;
@@ -1649,83 +1680,79 @@ obj_vasprintf(struct workspace *wk, struct sbuf *sb, const char *fmt, va_list ap
 				++fmt;
 			}
 
+			if (*fmt == 'o') {
+				obj o = va_arg(ap, unsigned int);
+				if (get_obj_type(wk, o) == obj_string && !quote_string) {
+					str_unescape(wk, sb, get_str(wk, o), false);
+				} else {
+					obj_to_s(wk, o, sb);
+				}
+
+				continue;
+			}
+
+			char fmt_buf[BUF_SIZE_1k + 1] = { 0 }; \
+			uint32_t len = fmt - fmt_start + 1; \
+			assert(len < BUF_SIZE_1k && "format specifier too long"); \
+			memcpy(fmt_buf, fmt_start, len);
+
 			switch (*fmt) {
 			case 'c':
 			case 'd': case 'i':
 				switch (int_len) {
-				case il_norm:
-					arg._int = va_arg(ap, int);
+				case il_norm: {
+					FMT_PARTIAL(va_arg(ap, int));
 					break;
-				case il_long:
-					arg._lint = va_arg(ap, long);
+				}
+				case il_long: {
+					FMT_PARTIAL(va_arg(ap, long));
 					break;
-				case il_long_long:
-					arg._llint = va_arg(ap, long long);
+				}
+				case il_long_long: {
+					FMT_PARTIAL(va_arg(ap, long long));
 					break;
 				}
+				}
 				break;
 			case 'u': case 'x': case 'X':
 				switch (int_len) {
-				case il_norm:
-					arg._uint = va_arg(ap, unsigned int);
+				case il_norm: {
+					FMT_PARTIAL(va_arg(ap, unsigned int));
+
 					break;
-				case il_long:
-					arg._luint = va_arg(ap, unsigned long);
+				}
+				case il_long: {
+					FMT_PARTIAL(va_arg(ap, unsigned long));
 					break;
-				case il_long_long:
-					arg._lluint = va_arg(ap, unsigned long long);
+				}
+				case il_long_long: {
+					FMT_PARTIAL(va_arg(ap, unsigned long long));
 					break;
 				}
+				}
 				break;
 			case 'e': case 'E':
 			case 'f': case 'F':
 			case 'g': case 'G':
-			case 'a': case 'A':
-				arg._double = va_arg(ap, double);
+			case 'a': case 'A': {
+				FMT_PARTIAL(va_arg(ap, double));
 				break;
-			case 's':
-				arg._charp = va_arg(ap, char *);
+			}
+			case 's': {
+				FMT_PARTIAL(va_arg(ap, char *));
 				break;
-			case 'p':
-				arg._voidp = va_arg(ap, void *);
+			}
+			case 'p': {
+				FMT_PARTIAL(va_arg(ap, void *));
 				break;
+			}
 			case 'n':
 			case '%':
 				break;
-			case 'o':
-				got_object = true;
-				obj = va_arg(ap, unsigned int);
-				break;
 			default:
 				assert(false && "unrecognized format");
 				break;
 			}
-
-			if (got_object) {
-				if (get_obj_type(wk, obj) == obj_string && !quote_string) {
-					str_unescape(wk, sb, get_str(wk, obj), false);
-				} else {
-					obj_to_s(wk, obj, sb);
-				}
-			} else {
-				char fmt_buf[BUF_SIZE_1k + 1] = { 0 };
-				uint32_t len = fmt - fmt_start + 1;
-				assert(len < BUF_SIZE_1k && "format specifier too long");
-				memcpy(fmt_buf, fmt_start, len);
-
-				// There is no portable way to create a
-				// va_list, so we have to enumerate all of the
-				// different possibilities
-				if (arg_width.have && arg_prec.have) {
-					sbuf_pushf(wk, sb, fmt_buf, arg_width.val, arg_prec.val, arg);
-				} else if (arg_width.have) {
-					sbuf_pushf(wk, sb, fmt_buf, arg_width.val, arg);
-				} else if (arg_prec.have) {
-					sbuf_pushf(wk, sb, fmt_buf, arg_prec.val, arg);
-				} else {
-					sbuf_pushf(wk, sb, fmt_buf, arg);
-				}
-			}
 		} else {
 			sbuf_push(wk, sb, *fmt);
 		}
