@@ -1,4 +1,10 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-FileCopyrightText: Simon Zeni <simon@bl4ckb0ne.ca>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <inttypes.h>
 #include <stdarg.h>
@@ -8,6 +14,7 @@
 #include "lang/eval.h"
 #include "lang/lexer.h"
 #include "lang/parser.h"
+#include "lang/string.h"
 #include "log.h"
 #include "tracy.h"
 
@@ -20,6 +27,7 @@ struct parser {
 	struct tokens *toks;
 	struct token *last_last, *last;
 	struct ast *ast;
+	struct workspace *wk;
 	uint32_t token_i, loop_depth, parse_depth;
 	bool caused_effect, valid, preserve_fmt_eol;
 
@@ -69,6 +77,10 @@ node_type_to_s(enum node_type t)
 static void
 parse_diagnostic(struct parser *p, struct token *err_tok, enum log_level lvl, const char *fmt, va_list args)
 {
+	if (p->mode & pm_quiet) {
+		return;
+	}
+
 	if (!err_tok) {
 		err_tok = p->last;
 	}
@@ -76,7 +88,7 @@ parse_diagnostic(struct parser *p, struct token *err_tok, enum log_level lvl, co
 	error_messagev(p->src, err_tok->line, err_tok->col, lvl, fmt, args);
 }
 
-__attribute__ ((format(printf, 3, 4)))
+MUON_ATTR_FORMAT(printf, 3, 4)
 static void
 parse_error(struct parser *p, struct token *err_tok, const char *fmt, ...)
 {
@@ -104,7 +116,13 @@ accept_comment(struct parser *p)
 {
 	if (p->last->type == tok_comment) {
 		struct node *n = darr_get(&p->ast->nodes, p->ast->nodes.len - 1);
-		n->comment = p->last->dat.s;
+
+		if (!n->comments.len) {
+			n->comments.start = p->ast->comments.len;
+		}
+
+		darr_push(&p->ast->comments, &p->last->dat.s);
+		++n->comments.len;
 		get_next_tok(p);
 	}
 }
@@ -264,9 +282,19 @@ node_to_s(struct node *n)
 	case node_argument:
 		i += snprintf(&buf[i], BUF_SIZE_S - i, ":%s", n->subtype == arg_kwarg ? "kwarg" : "normal");
 		break;
-	case node_if:
-		i += snprintf(&buf[i], BUF_SIZE_S - i, ":%s", n->subtype == (if_if || if_elseif) ? "normal" : "else");
+	case node_if: {
+		const char *l = NULL;
+		if (n->subtype == if_if) {
+			l = "if";
+		} else if (n->subtype == if_elseif) {
+			l = "elseif";
+		} else if (n->subtype == if_else) {
+			l = "else";
+		}
+
+		i += snprintf(&buf[i], BUF_SIZE_S - i, ":%s", l);
 		break;
+	}
 	case node_arithmetic: {
 		const char *s;
 		switch ((enum arithmetic_type)n->subtype) {
@@ -297,10 +325,6 @@ node_to_s(struct node *n)
 		break;
 	}
 
-	if (n->comment) {
-		i += snprintf(&buf[i], BUF_SIZE_S - i, " #%s", n->comment);
-	}
-
 	return buf;
 }
 
@@ -479,16 +503,31 @@ parse_e9(struct parser *p, uint32_t *id)
 	if (accept(p, tok_true)) {
 		n = make_node(p, id, node_bool);
 		n->subtype = 1;
+		if (p->wk) {
+			make_obj(p->wk, &n->l, obj_bool);
+			set_obj_bool(p->wk, n->l, n->subtype);
+		}
 	} else if (accept(p, tok_false)) {
 		n = make_node(p, id, node_bool);
 		n->subtype = 0;
+		if (p->wk) {
+			make_obj(p->wk, &n->l, obj_bool);
+			set_obj_bool(p->wk, n->l, n->subtype);
+		}
 	} else if (accept(p, tok_identifier)) {
 		make_node(p, id, node_id);
 	} else if (accept(p, tok_number)) {
-		make_node(p, id, node_number);
+		n = make_node(p, id, node_number);
+		if (p->wk) {
+			make_obj(p->wk, &n->l, obj_number);
+			set_obj_number(p->wk, n->l, n->dat.n);
+		}
 	} else if (accept(p, tok_string)) {
 		n = make_node(p, id, node_string);
 		n->subtype = p->last_last->n;
+		if (p->wk) {
+			n->l = make_strn(p->wk, n->dat.s, n->subtype);
+		}
 	} else {
 		make_node(p, id, node_empty);
 	}
@@ -978,9 +1017,9 @@ ret:
 }
 
 static bool
-parse_stmt(struct parser *p, uint32_t *id)
+parse_assignment(struct parser *p, uint32_t *id)
 {
-	uint32_t l_id = 0; // compiler thinks this won't get initialized...
+	uint32_t l_id = 0;
 	if (!(parse_expr(p, &l_id))) {
 		return false;
 	}
@@ -994,7 +1033,7 @@ parse_stmt(struct parser *p, uint32_t *id)
 		if (get_node(p->ast, l_id)->type != node_id) {
 			parse_error(p, NULL, "assignment target must be an id (got %s)", node_to_s(get_node(p->ast, l_id)));
 			return false;
-		} else if (!parse_stmt(p, &v)) {
+		} else if (!parse_expr(p, &v)) {
 			return false;
 		}
 
@@ -1017,7 +1056,7 @@ parse_stmt(struct parser *p, uint32_t *id)
 		if (get_node(p->ast, l_id)->type != node_id) {
 			parse_error(p, NULL, "assignment target must be an id (got %s)", node_to_s(get_node(p->ast, l_id)));
 			return false;
-		} else if (!parse_stmt(p, &v)) {
+		} else if (!parse_expr(p, &v)) {
 			return false;
 		}
 
@@ -1047,7 +1086,7 @@ parse_if(struct parser *p, uint32_t *id, enum if_type if_type)
 
 	struct token *if_ = p->last;
 	if (if_type == if_if || if_type == if_elseif) {
-		if (!parse_stmt(p, &cond_id)) {
+		if (!parse_expr(p, &cond_id)) {
 			return false;
 		}
 
@@ -1144,7 +1183,7 @@ parse_foreach(struct parser *p, uint32_t *id)
 
 	if (!expect(p, tok_colon)) {
 		return false;
-	} else if (!parse_stmt(p, &r_id)) {
+	} else if (!parse_expr(p, &r_id)) {
 		return false;
 	}
 
@@ -1247,7 +1286,7 @@ parse_line(struct parser *p, uint32_t *id)
 
 		make_node(p, id, node_break);
 	} else {
-		if (!parse_stmt(p, id)) {
+		if (!parse_assignment(p, id)) {
 			return false;
 		}
 	}
@@ -1319,7 +1358,7 @@ parse_block(struct parser *p, uint32_t *id)
 }
 
 bool
-parser_parse(struct ast *ast, struct source_data *sdata, struct source *src,
+parser_parse(struct workspace *wk, struct ast *ast, struct source_data *sdata, struct source *src,
 	enum parse_mode mode)
 {
 	TracyCZoneAutoS;
@@ -1341,9 +1380,15 @@ parser_parse(struct ast *ast, struct source_data *sdata, struct source *src,
 		.toks = &toks,
 		.valid = true,
 		.mode = mode,
+		.wk = wk,
 	};
 
 	darr_init(&ast->nodes, 2048, sizeof(struct node));
+
+	if (mode & pm_keep_formatting) {
+		darr_init(&ast->comments, 2048, sizeof(char *));
+	}
+
 	uint32_t id;
 	make_node(&parser, &id, node_null);
 	assert(id == 0);
@@ -1362,8 +1407,10 @@ ret:
 	TracyCZoneAutoE;
 	return ret;
 }
+
 void
 ast_destroy(struct ast *ast)
 {
 	darr_destroy(&ast->nodes);
+	darr_destroy(&ast->comments);
 }
