@@ -1,4 +1,11 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-FileCopyrightText: illiliti <illiliti@thunix.net>
+ * SPDX-FileCopyrightText: Simon Zeni <simon@bl4ckb0ne.ca>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -18,6 +25,7 @@
 #include "lang/interpreter.h"
 #include "lang/serial.h"
 #include "machine_file.h"
+#include "meson_opts.h"
 #include "options.h"
 #include "opts.h"
 #include "platform/mem.h"
@@ -43,7 +51,7 @@ load_obj_from_serial_dump(struct workspace *wk, const char *path, obj *res)
 {
 	bool ret = false;
 	FILE *f;
-	if (!(f = fs_fopen(path, "r"))) {
+	if (!(f = fs_fopen(path, "rb"))) {
 		return false;
 	}
 
@@ -148,7 +156,7 @@ cmd_exe(uint32_t argc, uint32_t argi, char *const argv[])
 		allocated_argv = true;
 	}
 
-	if (!run_cmd_argv(&ctx, opts.cmd[0], (char *const *)opts.cmd, envstr, envc)) {
+	if (!run_cmd_argv(&ctx, (char *const *)opts.cmd, envstr, envc)) {
 		LOG_E("failed to run command: %s", ctx.err_msg);
 		goto ret;
 	}
@@ -182,15 +190,21 @@ cmd_check(uint32_t argc, uint32_t argi, char *const argv[])
 	struct {
 		const char *filename;
 		bool print_ast;
+		uint32_t parse_mode;
 	} opts = { 0 };
 
-	OPTSTART("p") {
+	OPTSTART("pP") {
 		case 'p':
 			opts.print_ast = true;
 			break;
+		case 'P':
+			opts.print_ast = true;
+			opts.parse_mode = pm_keep_formatting;
+			break;
 	} OPTEND(argv[argi],
 		" <filename>",
-		"  -p - print parsed ast\n",
+		"  -p - print parsed ast\n"
+		"  -P - print parsed formatting ast\n",
 		NULL, 1)
 
 	opts.filename = argv[argi];
@@ -205,7 +219,7 @@ cmd_check(uint32_t argc, uint32_t argi, char *const argv[])
 		goto ret;
 	}
 
-	if (!parser_parse(&ast, &sdata, &src, 0)) {
+	if (!parser_parse(NULL, &ast, &sdata, &src, opts.parse_mode)) {
 		goto ret;
 	}
 
@@ -226,9 +240,14 @@ cmd_analyze(uint32_t argc, uint32_t argi, char *const argv[])
 {
 	struct analyze_opts opts = {
 		.replay_opts = error_diagnostic_store_replay_include_sources,
+		.enabled_diagnostics = analyze_diagnostic_unused_variable
+				       | analyze_diagnostic_dead_code,
 	};
 
-	OPTSTART("luqO:") {
+	OPTSTART("luqO:W:i:") {
+		case 'i':
+			opts.internal_file = optarg;
+			break;
 		case 'l':
 			opts.subdir_error = true;
 			opts.replay_opts &= ~error_diagnostic_store_replay_include_sources;
@@ -236,21 +255,66 @@ cmd_analyze(uint32_t argc, uint32_t argi, char *const argv[])
 		case 'O':
 			opts.file_override = optarg;
 			break;
-		case 'u':
-			opts.unused_variable_error = true;
-			break;
 		case 'q':
 			opts.replay_opts |= error_diagnostic_store_replay_errors_only;
 			break;
+		case 'W': {
+			bool enable = true;
+			const char *name = optarg;
+			if (str_startswith(&WKSTR(optarg), &WKSTR("no-"))) {
+				enable = false;
+				name += 3;
+			}
+
+			if (strcmp(name, "list") == 0) {
+				analyze_print_diagnostic_names();
+				return true;
+			} else if (strcmp(name, "error") == 0) {
+				opts.replay_opts |= error_diagnostic_store_replay_werror;
+			} else {
+				enum analyze_diagnostic d;
+				if (!analyze_diagnostic_name_to_enum(name, &d)) {
+					LOG_E("invalid diagnostic name '%s'", name);
+					return false;
+				}
+
+				if (enable) {
+					opts.enabled_diagnostics |= d;
+				} else {
+					opts.enabled_diagnostics &= ~d;
+				}
+			}
+			break;
+		}
 	} OPTEND(argv[argi], "",
 		"  -l - optimize output for editor linter plugins\n"
 		"  -q - only report errors\n"
-		"  -u - error on unused variables\n"
 		"  -O <path> - read project file with matching path from stdin\n"
+		"  -i <path> - analyze the single file <path> in internal mode\n"
+		"  -W [no-]<diagnostic> - enable or disable diagnostics\n"
+		"  -W list - list available diagnostics\n"
+		"  -W error - turn all warnings into errors\n"
 		,
 		NULL, 0)
 
-	return do_analyze(&opts);
+	if (opts.internal_file && opts.file_override) {
+		LOG_E("-i and -O are mutually exclusive");
+		return false;
+	}
+
+	SBUF_manual(abs);
+	if (opts.file_override) {
+		path_make_absolute(NULL, &abs, opts.file_override);
+		opts.file_override = abs.buf;
+	}
+
+	bool res;
+	res = do_analyze(&opts);
+
+	if (opts.file_override) {
+		sbuf_destroy(&abs);
+	}
+	return res;
 }
 
 static bool
@@ -265,8 +329,8 @@ cmd_options(uint32_t argc, uint32_t argi, char *const argv[])
 			opts.only_modified = true;
 			break;
 	} OPTEND(argv[argi], "",
-		"  -a - list all options"
-		"  -m - list only modified options"
+		"  -a - list all options\n"
+		"  -m - list only modified options\n"
 		,
 		NULL, 0)
 
@@ -679,7 +743,7 @@ cmd_test(uint32_t argc, uint32_t argi, char *const argv[])
 		return false;
 	}
 
-	return tests_run(&test_opts);
+	return tests_run(&test_opts, argv[0]);
 }
 
 static bool
@@ -720,7 +784,7 @@ cmd_setup(uint32_t argc, uint32_t argi, char *const argv[])
 			break;
 		case 'c': {
 			FILE *f;
-			if (!(f = fs_fopen(optarg, "r"))) {
+			if (!(f = fs_fopen(optarg, "rb"))) {
 				goto ret;
 			} else if (!serial_load(&wk, &wk.compiler_check_cache, f)) {
 				LOG_E("failed to load compiler check cache");
@@ -779,68 +843,81 @@ cmd_format(uint32_t argc, uint32_t argi, char *const argv[])
 	}
 
 	struct {
-		const char *filename;
+		char *const *filenames;
 		const char *cfg_path;
-		bool in_place;
+		bool in_place, check_only;
 	} opts = { 0 };
 
-	OPTSTART("ic:") {
+	OPTSTART("ic:q") {
 		case 'i':
 			opts.in_place = true;
 			break;
 		case 'c':
 			opts.cfg_path = optarg;
 			break;
-	} OPTEND(argv[argi], " <filename>",
-		"  -i - modify file in-place\n"
-		"  -c <cfg.ini> - read configuration from cfg\n",
-		NULL, 1)
+		case 'q':
+			opts.check_only = true;
+			break;
+	} OPTEND(argv[argi], " <file>[ <file>[...]]",
+		"  -q - exit with 1 if files would be modified by muon fmt\n"
+		"  -i - format files in-place\n"
+		"  -c <muon_fmt.ini> - read configuration from muon_fmt.ini\n",
+		NULL, -1)
 
-	opts.filename = argv[argi];
-
-	bool opened_out = false;
-	bool ret = false;
-
-	struct source src = { 0 };
-	struct ast ast = { 0 };
-	struct source_data sdata = { 0 };
-
-	if (!fs_read_entire_file(opts.filename, &src)) {
-		goto ret;
-	} else if (!parser_parse(&ast, &sdata, &src, pm_keep_formatting)) {
-		goto ret;
+	if (opts.in_place && opts.check_only) {
+		LOG_E("-q and -i are mutually exclusive");
+		return false;
 	}
 
+	opts.filenames = &argv[argi];
+	const uint32_t num_files = argc - argi;
+
+	bool ret = true;
+	bool opened_out;
 	FILE *out;
+	uint32_t i;
+	for (i = 0; i < num_files; ++i) {
+		bool fmt_ret = true;
+		opened_out = false;
 
-	if (opts.in_place) {
-		if (!(out = fs_fopen(opts.filename, "w"))) {
-			goto ret;
+		struct source src = { 0 };
+		if (!fs_read_entire_file(opts.filenames[i], &src)) {
+			ret = false;
+			goto cont;
 		}
-		opened_out = true;
-	} else {
-		out = stdout;
+
+		if (opts.in_place) {
+			if (!(out = fs_fopen(opts.filenames[i], "wb"))) {
+				ret = false;
+				goto cont;
+			}
+			opened_out = true;
+		} else if (opts.check_only) {
+			out = NULL;
+		} else {
+			out = stdout;
+		}
+
+		fmt_ret = fmt(&src, out, opts.cfg_path, opts.check_only);
+cont:
+		if (opened_out) {
+			fs_fclose(out);
+
+			if (!fmt_ret) {
+				fs_write(opts.filenames[i], (const uint8_t *)src.src, src.len);
+			}
+		}
+		fs_source_destroy(&src);
+		ret &= fmt_ret;
 	}
 
-	if (!fmt(&ast, out, opts.cfg_path)) {
-		goto ret;
-	}
-
-	ret = true;
-ret:
-	if (opened_out) {
-		fs_fclose(out);
-	}
-	fs_source_destroy(&src);
-	ast_destroy(&ast);
-	source_data_destroy(&sdata);
 	return ret;
 }
 
 static bool
 cmd_version(uint32_t argc, uint32_t argi, char *const argv[])
 {
-	printf("muon v%s%s%s\nmeson compatibility version %s\nenabled features:\n",
+	printf("muon %s%s%s\nmeson compatibility version %s\nenabled features:\n",
 		muon_version.version,
 		*muon_version.vcs_tag ? "-" : "",
 		muon_version.vcs_tag,
@@ -870,6 +947,33 @@ cmd_version(uint32_t argc, uint32_t argi, char *const argv[])
 	return true;
 }
 
+static bool cmd_main(uint32_t argc, uint32_t argi, char *argv[]);
+
+static bool
+cmd_meson(uint32_t argc, uint32_t argi, char *const argv[])
+{
+	++argi;
+
+	struct workspace wk;
+	char **new_argv;
+	uint32_t new_argc, new_argi;
+	workspace_init_bare(&wk);
+	if (!translate_meson_opts(&wk, argc, argi, (char **)argv, &new_argc, &new_argi, &new_argv)) {
+		return false;
+	}
+
+	argi = new_argi;
+	argc = new_argc;
+	argv = new_argv;
+
+	bool res = cmd_main(argc, argi, (char **)argv);
+
+	workspace_destroy(&wk);
+	z_free(new_argv);
+
+	return res;
+}
+
 static bool
 cmd_main(uint32_t argc, uint32_t argi, char *argv[])
 {
@@ -882,6 +986,7 @@ cmd_main(uint32_t argc, uint32_t argi, char *argv[])
 		{ "info", cmd_info, NULL },
 		{ "install", cmd_install, "install project" },
 		{ "internal", cmd_internal, "internal subcommands" },
+		{ "meson", cmd_meson, "meson compatible cli proxy" },
 		{ "options", cmd_options, "list project options" },
 		{ "samu", cmd_samu, have_samurai ? "run samurai" : NULL },
 		{ "setup", cmd_setup, "setup a build directory" },
@@ -939,7 +1044,23 @@ main(int argc, char *argv[])
 
 	compilers_init();
 
-	int ret = cmd_main(argc, 0, argv) ? 0 : 1;
+	bool res;
+	bool meson_compat = false;
+
+	{
+		SBUF(basename);
+		path_basename(NULL, &basename, argv[0]);
+		meson_compat = strcmp(basename.buf, "meson") == 0 && strcmp(argv[1], "internal") != 0;
+		sbuf_destroy(&basename);
+	}
+
+	if (meson_compat) {
+		res = cmd_meson(argc, 0, argv);
+	} else {
+		res = cmd_main(argc, 0, argv);
+	}
+
+	int ret = res ? 0 : 1;
 
 	path_deinit();
 	return ret;
