@@ -1,4 +1,9 @@
-#include "posix.h"
+/*
+ * SPDX-FileCopyrightText: Stone Tickle <lattis@mochiro.moe>
+ * SPDX-License-Identifier: GPL-3.0-only
+ */
+
+#include "compat.h"
 
 #include <inttypes.h>
 #include <stdlib.h>
@@ -15,7 +20,7 @@
 #include "platform/filesystem.h"
 #include "platform/path.h"
 
-bool created_options_are_builtin = false;
+bool initializing_builtin_options = false;
 
 static const char *build_option_type_to_s[build_option_type_count] = {
 	[op_string] = "string",
@@ -82,7 +87,6 @@ parse_config_string(struct workspace *wk, const struct str *ss, struct option_ov
 	oo->val = make_strn(wk, val.s, val.len);
 	if (have_subproject) {
 		oo->proj = make_strn(wk, subproject.s, subproject.len);
-		obj_fprintf(wk, log_file(), "subproject option override: %o:%o=%o\n", oo->proj, oo->name, oo->val);
 	}
 
 	return true;
@@ -120,7 +124,7 @@ check_invalid_subproject_option(struct workspace *wk)
 
 	for (i = 0; i < wk->option_overrides.len; ++i) {
 		oo = darr_get(&wk->option_overrides, i);
-		if (!oo->proj) {
+		if (!oo->proj || oo->source < option_value_source_commandline) {
 			continue;
 		}
 
@@ -387,6 +391,48 @@ typecheck_opt(struct workspace *wk, uint32_t err_node, obj val, enum build_optio
 	return true;
 }
 
+struct prefix_dir_opts_ctx {
+	const struct str *prefix;
+};
+
+static enum iteration_result
+prefix_dir_opts_iter(struct workspace *wk, void *_ctx, obj _k, obj v)
+{
+	struct prefix_dir_opts_ctx *ctx = _ctx;
+
+	struct obj_option *opt = get_obj_option(wk, v);
+
+	if (opt->kind != build_option_kind_prefixed_dir) {
+		return ir_cont;
+	}
+
+	const char *path = get_cstr(wk, opt->val);
+	if (!path_is_absolute(path)) {
+		return ir_cont;
+	}
+
+	SBUF(new_path);
+	if (path_is_subpath(ctx->prefix->s, path)) {
+		path_relative_to(wk, &new_path, ctx->prefix->s, path);
+	} else {
+		path_join(wk, &new_path, ctx->prefix->s, path);
+	}
+
+	opt->val = sbuf_into_str(wk, &new_path);
+	return ir_cont;
+}
+
+bool
+prefix_dir_opts(struct workspace *wk)
+{
+	obj prefix;
+	get_option_value(wk, NULL, "prefix", &prefix);
+
+	return obj_dict_foreach(wk, wk->global_opts, &(struct prefix_dir_opts_ctx) {
+		.prefix = get_str(wk, prefix),
+	}, prefix_dir_opts_iter);
+}
+
 static void
 extend_array_option(struct workspace *wk, obj opt, obj new_val,
 	enum option_value_source source)
@@ -511,7 +557,7 @@ create_option(struct workspace *wk, uint32_t node, obj opts, obj opt, obj val)
 
 	struct obj_option *o = get_obj_option(wk, opt);
 
-	if (created_options_are_builtin) {
+	if (initializing_builtin_options) {
 		o->builtin = true;
 	}
 
@@ -602,7 +648,6 @@ set_compile_opt_from_env(struct workspace *wk, const char *name, const char *fla
 		UNREACHABLE;
 	}
 
-
 	if ((flags = getenv(flags)) && *flags) {
 		extend_array_option(wk, opt, str_split(wk, &WKSTR(flags), NULL),
 			option_value_source_environment);
@@ -614,6 +659,20 @@ set_compile_opt_from_env(struct workspace *wk, const char *name, const char *fla
 	}
 }
 
+static void
+set_str_opt_from_env(struct workspace *wk, const char *env_name, const char *opt_name)
+{
+	obj opt;
+	if (!get_option(wk, NULL, &WKSTR(opt_name), &opt)) {
+		UNREACHABLE;
+	}
+
+	const char *env_val;
+	if ((env_val = getenv(env_name)) && *env_val) {
+		set_option(wk, 0, opt, make_str(wk, env_val), option_value_source_environment, false);
+	}
+}
+
 static bool
 init_builtin_options(struct workspace *wk, const char *script, const char *fallback)
 {
@@ -625,9 +684,9 @@ init_builtin_options(struct workspace *wk, const char *script, const char *fallb
 	enum language_mode old_mode = wk->lang_mode;
 	wk->lang_mode = language_opts;
 	obj _;
-	created_options_are_builtin = true;
+	initializing_builtin_options = true;
 	bool ret = eval_str(wk, opts, eval_mode_default, &_);
-	created_options_are_builtin = false;
+	initializing_builtin_options = false;
 	wk->lang_mode = old_mode;
 	return ret;
 }
@@ -690,7 +749,7 @@ setup_project_options(struct workspace *wk, const char *cwd)
 	if (fs_file_exists(meson_opts.buf)) {
 		enum language_mode old_mode = wk->lang_mode;
 		wk->lang_mode = language_opts;
-		if (!wk->eval_project_file(wk, meson_opts.buf)) {
+		if (!wk->eval_project_file(wk, meson_opts.buf, false)) {
 			return false;
 		}
 		wk->lang_mode = old_mode;
@@ -743,25 +802,34 @@ init_global_options(struct workspace *wk)
 		"option('prefix', type: 'string', value: '/usr/local')\n"
 		"option('bindir', type: 'string', value: 'bin')\n"
 		"option('mandir', type: 'string', value: 'share/man')\n"
+		"option('datadir', type: 'string', value: 'share')\n"
+		"option('libdir', type: 'string', value: 'lib')\n"
+		"option('includedir', type: 'string', value: 'include')\n"
 		"option('wrap_mode', type: 'string', value: 'nopromote')\n"
 		"option('force_fallback_for', type: 'array', value: [])\n"
 		"option('pkg_config_path', type: 'string', value: '')\n"
 		"option('c_args', type: 'array', value: [])\n"
 		"option('c_link_args', type: 'array', value: [])\n"
+		"option('werror', type: 'boolean', value: false)\n"
 
 		"option('env.CC', type: 'array', value: ['cc'])\n"
+		"option('env.NINJA', type: 'array', value: ['ninja'])\n"
 		)) {
 		return false;
 	}
 
 	set_binary_from_env(wk, "CC", "env.CC");
+	set_binary_from_env(wk, "NINJA", "env.NINJA");
 	set_compile_opt_from_env(wk, "c_args", "CFLAGS", "CPPFLAGS");
 	set_compile_opt_from_env(wk, "c_link_args", "CFLAGS", "LDFLAGS");
 
 #ifdef MUON_BOOTSTRAPPED
 	set_binary_from_env(wk, "CXX", "env.CXX");
+	set_binary_from_env(wk, "OBJC", "env.OBJC");
+	set_binary_from_env(wk, "NASM", "env.NASM");
 	set_compile_opt_from_env(wk, "cpp_args", "CXXFLAGS", "CPPFLAGS");
 	set_compile_opt_from_env(wk, "cpp_link_args", "CXXFLAGS", "LDFLAGS");
+	set_str_opt_from_env(wk, "PKG_CONFIG_PATH", "pkg_config_path");
 #endif
 
 	return true;
@@ -1027,7 +1095,7 @@ list_options_iter(struct workspace *wk, void *_ctx, obj key, obj val)
 		no_clr = "\033[0m";
 	}
 
-	obj_printf(wk, "  %s%#o%s=", key_clr, key, no_clr);
+	obj_fprintf(wk, log_file(), "  %s%#o%s=", key_clr, key, no_clr);
 
 	obj choices = 0;
 	obj selected = 0;
@@ -1089,33 +1157,33 @@ list_options_iter(struct workspace *wk, void *_ctx, obj key, obj val)
 	case op_boolean:
 	case op_combo:
 	case op_feature:
-		obj_printf(wk, "<%s>", get_cstr(wk, choices));
+		obj_fprintf(wk, log_file(), "<%s>", get_cstr(wk, choices));
 		break;
 	case op_string: {
 		const struct str *def = get_str(wk, opt->val);
-		obj_printf(wk, "<%s>, default: %s%s%s", get_cstr(wk, choices),
+		obj_fprintf(wk, log_file(), "<%s>, default: %s%s%s", get_cstr(wk, choices),
 			sel_clr, def->len ? def->s : "''", no_clr);
 		break;
 	}
 	case op_integer:
-		printf("<%sN%s>", val_clr, no_clr);
+		log_plain("<%sN%s>", val_clr, no_clr);
 		if (opt->min || opt->max) {
-			printf(" where ");
+			log_plain(" where ");
 			if (opt->min) {
-				obj_printf(wk, "%o <= ", opt->min);
+				obj_fprintf(wk, log_file(), "%o <= ", opt->min);
 			}
-			printf("%sN%s", val_clr, no_clr);
+			log_plain("%sN%s", val_clr, no_clr);
 			if (opt->max) {
-				obj_printf(wk, " <= %o", opt->max);
+				obj_fprintf(wk, log_file(), " <= %o", opt->max);
 			}
 		}
-		obj_printf(wk, ", default: %s%o%s", sel_clr, opt->val, no_clr);
+		obj_fprintf(wk, log_file(), ", default: %s%o%s", sel_clr, opt->val, no_clr);
 
 		break;
 	case op_array:
-		printf("<%svalue%s[,%svalue%s[...]]>", val_clr, no_clr, val_clr, no_clr);
+		log_plain("<%svalue%s[,%svalue%s[...]]>", val_clr, no_clr, val_clr, no_clr);
 		if (opt->choices) {
-			obj_printf(wk, " where value in %s", get_cstr(wk, choices));
+			obj_fprintf(wk, log_file(), " where value in %s", get_cstr(wk, choices));
 		}
 		break;
 	default:
@@ -1123,14 +1191,14 @@ list_options_iter(struct workspace *wk, void *_ctx, obj key, obj val)
 	}
 
 	if (opt->source != option_value_source_default) {
-		obj_printf(wk, "*");
+		obj_fprintf(wk, log_file(), "*");
 	}
 
 	if (opt->description) {
-		obj_printf(wk, "\n    %#o", opt->description);
+		obj_fprintf(wk, log_file(), "\n    %#o", opt->description);
 	}
 
-	printf("\n");
+	log_plain("\n");
 	return ir_cont;
 }
 
@@ -1151,7 +1219,7 @@ list_options(const struct list_options_opts *list_opts)
 		path_make_absolute(&wk, &meson_opts, "meson_options.txt");
 
 		if (fs_file_exists(meson_opts.buf)) {
-			if (!wk.eval_project_file(&wk, meson_opts.buf)) {
+			if (!wk.eval_project_file(&wk, meson_opts.buf, false)) {
 				goto ret;
 			}
 		}
@@ -1165,23 +1233,13 @@ list_options(const struct list_options_opts *list_opts)
 	} else {
 		SBUF(option_info);
 		path_join(&wk, &option_info, output_path.private_dir, output_path.option_info);
-
 		if (!fs_file_exists(option_info.buf)) {
 			LOG_I("run this command must be run from a build directory or the project root");
 			goto ret;
 		}
 
-		FILE *f;
-		if (!(f = fs_fopen(option_info.buf, "rb"))) {
-			goto ret;
-		}
-
 		obj arr;
-		if (!serial_load(&wk, &arr, f)) {
-			goto ret;
-		}
-
-		if (!fs_fclose(f)) {
+		if (!serial_load_from_private_dir(&wk, &arr, output_path.option_info)) {
 			goto ret;
 		}
 
@@ -1192,24 +1250,24 @@ list_options(const struct list_options_opts *list_opts)
 	struct list_options_ctx ctx = { .list_opts = list_opts };
 
 	if (get_obj_dict(&wk, current_project(&wk)->opts)->len) {
-		printf("project options:\n");
+		log_plain("project options:\n");
 		obj_dict_foreach(&wk, current_project(&wk)->opts, &ctx, list_options_iter);
 	} else if (!list_opts->list_all) {
-		printf("no project options defined\n");
+		log_plain("no project options defined\n");
 	}
 
 	if (list_opts->list_all) {
 		if (get_obj_dict(&wk, current_project(&wk)->opts)->len) {
-			printf("\n");
+			log_plain("\n");
 		}
 
 		ctx.show_builtin = true;
 
-		printf("project builtin-options:\n");
+		log_plain("project builtin-options:\n");
 		obj_dict_foreach(&wk, current_project(&wk)->opts, &ctx, list_options_iter);
-		printf("\n");
+		log_plain("\n");
 
-		printf("global options:\n");
+		log_plain("global options:\n");
 		obj_dict_foreach(&wk, wk.global_opts, &ctx, list_options_iter);
 	}
 
